#ifndef RADIOAPI_CPP
#define RADIOAPI_CPP

/*
 *
 * Radio API source module for PCnet-MOBILE 802.11 protocol firmware.
 *
 * Copyright (c) 1996, 1997, 1998 Advanced Micro Devices, Inc.
 * All rights reserved.
 *
 * Version 2.01
 *
 * Controller & Switch Development (CSD)
 *
 * Module Name: radioapi.cpp
 *
 * Globals Accessed:
 *
 * Callers:
 *
 * Called routines:
 *
 * Functional Description:
 *
 * Comments:
 *
 * Radio API Version 005
 * - Modified the routine PLCP_Header_Type::get_length() to compare with the
 * 2Mb value instead of the 1Mb value to be consistent with the way the
 * get_PLCP_info routine operates.  If not a miscalculation may occur and
 * throw the buffer management routines off.
 *
 * - Modified the get_plcp_info routine to set the rate value to the 1Mb
 * constant if the incoming frame was not the value indicating 2Mb.
 *
 * Radio API Version 004
 * Modified the initialization of the 3824 code so as to not program register
 * ..number 40, which does not exist.
 * Applied a conditional assembly statement in the get_RSSI routine in the file
 * ..api_asm.asm to facilitate use of different 3824 register if so desired.
 * ..The default register is the RSSI register.
 *
 * Radio API Version 003
 * Modified programming of the 3824 to use the PLCP information to assert
 * ..carrier for the duration of a receive frame.  Change will have NO effect
 * ..on HSP3824 part.
 *
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 *              THESE CHANGES ARE NOT BACKWARD COMPATIBLE
 *
 * Firmware
 * --------
 * Modified PLCP to comply with 802.11 standard.  Value in length field was
 * ..expressed as the number of bytes and is now expressed as the duration
 * ..in microseconds of the MPDU.  The get_PLCP_info and build_PLCP_header
 * ..were modified and the set_length was replaced with a rate specific
 * ..routines to support this change.
 *
 * Harris DSSS baseband processor
 * ------------------------------
 * Modified programming of the 3824 to invert the Q signal on transmit and
 * ..recieve of DQPSK to comply with 802.11 standard.  Change will effect
 * ..the HFA3824A part only.  This programming will have no effect on the
 * ..HFA3824 and HSP3824 parts!
 *
 * PLEASE NOTE:  a radio with a HFA3824A part programmed with this code
 * ..will not be able to communicate with a radio with either a HFA3824 or
 * ..HSP3824 part.
 *
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * Radio API Version 002
 * Removed link LED toggle from force_channel routine.
 * Modified user_function to examine driver_state byte.
 *
 * Radio API Version 001
 * Modified which 3824 register is read by save_RSSI call from signal quality
 * ..register to RSSI.
 * Added routine get_hdw_delay ().
 *
 * Radio API Version 000
 * Added radio API version field.
 *
 * Diagnostics:
 *
 */


#include <dos.h>
#include "register.hpp"
#include "radio.hpp"
#include "radioapi.hpp"

extern const Unsigned8 Crc_Table_LSB[256];
extern const Unsigned8 Crc_Table_MSB[256];

void pks_dummy(void);  //  time delay routine

Unsigned8 Sleep_TIR0_mode0=0x00;

Unsigned8 Sleep_TIR0_mode1=0x00;

Unsigned8 Sleep_TIR0_mode2=0x00;
Unsigned8 Sleep_TIR11_mode2=0x00;
Unsigned8 Sleep_TCR7_mode2=0x00;
Unsigned8 Sleep_TCR15_mode2=0x00;
Unsigned8 Sleep_TCR30_mode2=0x00;

int Sleep_Level=0;

Preamble_Type  pre;

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
Preamble_Type::Preamble_Type()
{
  int i= 0;    //  Need to initialize i, to work around C++ compiler bug.

  for (i=0; i<PREAMBLE_LEN; i++)
  {
#ifdef COAX
    preamble[i] = 0xAA;
#else
    preamble[i] = 0xFF;
#endif
  }
  SFD = (SFD_B1 << 8) | SFD_B0;
}

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#ifdef COAX
void Preamble_Type::set_SFD(Unsigned8 channel)
{
  SFD = (SFD_B1 << 8) | channel;
}
#endif  // COAX




//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
PLCP_Header_Type::PLCP_Header_Type()
{
  // Initialize the default (1Mbps) PLCP

    ds_signal = BPSK_SIGNAL;      // This signal value indicates BPSK (1Mbps)
    ds_service = DS_PLCP_SERVICE;
    ds_length = 0;
    ds_error_check = 0;

}  // End default (1Mbps) PLCP_Header_Type constructor


//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline void PLCP_Header_Type::set_signal(Unsigned8 rate)
{
  if (rate != ONE_MBIT) {
    // 2Mbps
    ds_signal = QPSK_SIGNAL;    // This signal value indicates QPSK (2Mbps)
  } else {
      // 1Mbps
#    ifdef FORCE_QPSK
      ds_signal = QPSK_SIGNAL;  // This signal value indicates QPSK (2Mbps)
#    else
      ds_signal = BPSK_SIGNAL;  // This signal value indicates BPSK (1Mbps)
#    endif /* end FORCE_QPSK */
  } /* end if */
} /* end set_signal */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline void PLCP_Header_Type::set_service(Unsigned8 rate)
{
  ds_service = DS_PLCP_SERVICE;
} /* end set_service */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline void PLCP_Header_Type::set_length2Mb(Unsigned16 length)
{
  // Set the length field in the (Any Mbps) PLCP
// The 'length' field in the DS PLCP is in fact a duration which indicates
// ..the number of microseconds required to transmit the MPDU.  Here the
// ..number of bytes is multiplied by 4 since each byte takes 4 microseconds
// ..to transmit.

  ds_length = length * 4 ;
} /* end set_length */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline void PLCP_Header_Type::set_length1Mb(Unsigned16 length)
{
  // Set the length field in the (Any Mbps) PLCP
// The 'length' field in the DS PLCP is in fact a duration which indicates
// ..the number of microseconds required to transmit the MPDU.  Here the
// ..number of bytes is multiplied by 8 since each byte takes 8 microseconds
// ..to transmit.

  ds_length = length * 8 ;
} /* end set_length */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline void PLCP_Header_Type::set_EC()
{
  // Set the error_check field in the (either Mbps) PLCP
  ds_error_check = CRC_16();
} /* end set_EC */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline const Unsigned16 PLCP_Header_Type::CRC_16()
{
  register Unsigned16 table_index;
  register Unsigned16 crc;

/*  Original FHSS CRC algorithm code
  table_index = Unsigned8(~((Unsigned8*) this)[0]);
  ((Unsigned8*) &crc)[1] = ~Crc_Table_MSB[table_index];
  ((Unsigned8*) &crc)[0] = Crc_Table_LSB[table_index];

  table_index = ((Unsigned8*) &crc)[1] ^ ((Unsigned8*) this)[1];
  ((Unsigned8*) &crc)[1] = ((Unsigned8*) &crc)[0] ^ Crc_Table_MSB[table_index];
  ((Unsigned8*) &crc)[0] = Crc_Table_LSB[table_index];

  return (((~crc) << 8) | ((~crc) >> 8));
*/

  // The following code was added to support CRC16 for the 6-byte DSSS PLCP

  // Test the Signal field in (this) DS PLCP
  if ( ((Unsigned8*) this)[0] == BPSK_SIGNAL)  {
    crc = CRC16_MACRO((CRC16_MACRO(0xFFFF,BPSK_SIGNAL)),DS_PLCP_SERVICE);
//    crc = 0x0DC8;
    // Constant CRC value up to length field for BPSK
  } else {
    crc = CRC16_MACRO((CRC16_MACRO(0xFFFF,QPSK_SIGNAL)),DS_PLCP_SERVICE);
//    crc = 0x0249;
    // Constant CRC value up to length field for QPSK
  } /* end if */

  table_index = ((Unsigned8*) &crc)[0] ^ ((Unsigned8*) this)[2];
  ((Unsigned8*) &crc)[0] = ((Unsigned8*) &crc)[1] ^ Crc_Table_MSB[table_index];
  ((Unsigned8*) &crc)[1] = Crc_Table_LSB[table_index];

  table_index = ((Unsigned8*) &crc)[0] ^ ((Unsigned8*) this)[3];
  ((Unsigned8*) &crc)[0] = ((Unsigned8*) &crc)[1] ^ Crc_Table_MSB[table_index];
  ((Unsigned8*) &crc)[1] = Crc_Table_LSB[table_index];

  return (~crc);          // Return 1's complement of the CRC
} /* end CRC_16 */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline const Unsigned16 PLCP_Header_Type::get_length() const
{
  return (ds_signal == TWO_MBIT) ? ds_length >> 2 : ds_length >> 3 ;
  //return (ds_length);
} /* end get_length */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline const Unsigned8 PLCP_Header_Type::get_rate() const
{
  return (ds_signal);

} /* end get_rate */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
inline const Unsigned16 PLCP_Header_Type::get_EC() const
{
  return (ds_error_check);
} /* end get_EC */







//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
/*  The following routines are the methods defined in the Radio_API class.
 *  These class methods must be supported for all PHY types, even if
 *  the routine is empty.
 */


Unsigned8 Next_TX_Cntl;            // Set by set_PHY_rate()
                      // Used by arm_tx(), enable_tx()
                      // and enable_tx_if_good()
Unsigned8 Next_Response_TX_Cntl;      // Set by get_PLCP_info()
                      // based on the rx frame rate.
                      // Used by enable_response_if_good()
                      // to set the rate for the response
                      // frame.
Unsigned16 rel_time_correction_factor;    // This variable is used by
                      // rel_time_to_usec to remember the
                      // sub-microsecond remainder between
                      // calls. It is assumed that
                      // rel_time_to_usec is only called
                      // by Update_Time in Sys_Timer so
                      // that the remainder can be used
                      // and updated on each call.
//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Boolean Radio_API_Module::initialize(int reg_domain, char far* reg_domain_bytes) {

// write out the radio API version information
#define kAPIVersionOffset 28

  unsigned int
    uiSize = sizeof(TIR) +
             sizeof(MIR) +
             sizeof(Reserved_1) +
             kAPIVersionOffset ;

  char far
    * pch0 = (char far *)&TIR + uiSize ;

  * pch0 = '0' ;
  (char far *)pch0 ++ ;
  * pch0 = '0' ;
  (char far *)pch0 ++ ;
  * pch0 = '5' ;


  TIR[NETWORK_CONTROL] = SRES;                      // TIR0
  TIR[NETWORK_CONTROL] = 0x00;

  TIR[SERIAL_DEVICE] = SDSEL3;                      // TIR2
  TIR[INT_REG_1] = 0xFFFF;                          // TIR4
  TIR[INT_UNMASK_1] = 0x00;                         // TIR6
  TIR[TXMT_CONTROL] = TXRES | TXFR;                 // TIR8
  TIR[TXMT_CONTROL] = 0;
  TIR[TXMT_SEQ_CNTL] = RCEN;                        // TIR11
  TIR[RCVR_CONTROL] = RXRES | RXFR;                 // TIR16
  TIR[RCVR_CONTROL] = 0;
  TIR[ANT_DVRSTY_A_D] = 0x08;                       // TIR26
  TIR[RSSI_LOW_LMT] = 0x00;                         // TIR28

#if defined (HARDWARE_DEBUG_PIN) && !defined(IMAX)
  TIR[USER_PIN_DATA] = 0x19;                        // TIR29
                                                    // Drives USER0 (debug pin) HIGH (inactive)
                                                    // Drives USER4, 3, 0 = 1 & USER2, 1 = 0
#else
  TIR[USER_PIN_DATA] = 0x18;                        // TIR29
                                                    // Drives USER4, 3 = 1 & USER2, 1, 0 = 0
#endif  // HARDWARE_DEBUG_PIN

  TIR[CONFIG_INDEX] = NETWORK_CONFIGURATION;        // TCR0
  TIR[CONFIG_DATA_PORT] = DRB(2) | HDB(0) | SD(2);

  TIR[CONFIG_INDEX] = TRANSMIT_CONFIGURATION;       // TCR1
  TIR[CONFIG_DATA_PORT] = TXDC_1 | TXDC_0;

  TIR[CONFIG_INDEX] = CLOCK_RECOVERY;               // TCR2
#ifdef COAX
  TIR[CONFIG_DATA_PORT] = CLKP(0);
#else
  TIR[CONFIG_DATA_PORT] = ECLK | CLKP(5);
#endif

  TIR[CONFIG_INDEX] = RECEIVE_CONFIGURATION;        // TCR3
  TIR[CONFIG_DATA_PORT] = PFL(PLCP_LEN);

  TIR[CONFIG_INDEX] = ANTENNA_DIVERSITY_TIMER;      // TCR4
  TIR[CONFIG_DATA_PORT] = ADT(0);

  TIR[CONFIG_INDEX] = TX_RAMP_UP_TIMING;            // TCR5
  TIR[CONFIG_DATA_PORT] = TGAP1(0) | TGAP2(2);

  TIR[CONFIG_INDEX] = TX_RAMP_DOWN_TIMING;          // TCR6
  TIR[CONFIG_DATA_PORT] = TGAP3(2) | TGAP4(0);

  TIR[CONFIG_INDEX] = PIN_DATA_A;                   // TCR7
  TIR[CONFIG_DATA_PORT] = 0x02;

  TIR[CONFIG_INDEX] = START_DELIMITER_LSB;          // TCR8
  TIR[CONFIG_DATA_PORT] = 0x00;

  TIR[CONFIG_INDEX] = START_DELIMITER_CSB;          // TCR9
  TIR[CONFIG_DATA_PORT] = SFD_B0;

  TIR[CONFIG_INDEX] = START_DELIMITER_MSB;          // TCR10
  TIR[CONFIG_DATA_PORT] = SFD_B1;

  TIR[CONFIG_INDEX] = INTERRUPT_REGISTER_3;         // TCR11
  TIR[CONFIG_DATA_PORT] = 0xFF;

  TIR[CONFIG_INDEX] = INTERRUPT_UNMASK_REG_3;       // TCR12
  TIR[CONFIG_DATA_PORT] = 0x00;

  // TCR13 done later after all logic levels have been defined

  // TCR14 done later after all logic levels have been defined

  TIR[CONFIG_INDEX] = PIN_CONFIGURATION_C;          // TCR15
  TIR[CONFIG_DATA_PORT] = 0x82;

#ifdef COAX    // COAX PHY (no DS radio)

  TIR[CONFIG_INDEX] = BAUD_DETECT_START;            // TCR16
  TIR[CONFIG_DATA_PORT] = BDS(15);

  TIR[CONFIG_INDEX] = BAUD_DETECT_LOWER_LIMIT;      // TCR17
  TIR[CONFIG_DATA_PORT] = BDLLT(20);

  TIR[CONFIG_INDEX] = BAUD_DETECT_UPPER_LIMIT;      // TCR18
  TIR[CONFIG_DATA_PORT] = BDULT(20);

  TIR[CONFIG_INDEX] = BAUD_DTCT_ACCEPT_CNT_CS;      // TCR19
  TIR[CONFIG_DATA_PORT] = BDCS(8);

  TIR[CONFIG_INDEX] = BAUD_DTCT_ACCEPT_CNT_SD;      // TCR20
  TIR[CONFIG_DATA_PORT] = BDSD(8);

  TIR[CONFIG_INDEX] = BAUD_DETECT_RATIO;            // TCR21
  TIR[CONFIG_DATA_PORT] = BDRN(4);

  TIR[CONFIG_INDEX] = BAUD_DETECT_ACCEPT_COUNT;     // TCR22
  TIR[CONFIG_DATA_PORT] = 0x00;

  TIR[CONFIG_INDEX] = BAUD_DETECT_FAIL_COUNT;       // TCR23
  TIR[CONFIG_DATA_PORT] = 0x00;

#else    // DS radio build

  TIR[CONFIG_INDEX] = BAUD_DETECT_START;            // TCR16
  TIR[CONFIG_DATA_PORT] = BDS(0);

  TIR[CONFIG_INDEX] = BAUD_DETECT_LOWER_LIMIT;      // TCR17
  TIR[CONFIG_DATA_PORT] = BDLLT(0);

  TIR[CONFIG_INDEX] = BAUD_DETECT_UPPER_LIMIT;      // TCR18
  TIR[CONFIG_DATA_PORT] = BDULT(0);

  TIR[CONFIG_INDEX] = BAUD_DTCT_ACCEPT_CNT_CS;      // TCR19
  TIR[CONFIG_DATA_PORT] = BDCS(0);

  TIR[CONFIG_INDEX] = BAUD_DTCT_ACCEPT_CNT_SD;      // TCR20
  TIR[CONFIG_DATA_PORT] = BDSD(0);

  TIR[CONFIG_INDEX] = BAUD_DETECT_RATIO;            // TCR21
  TIR[CONFIG_DATA_PORT] = BDRN(0);

  TIR[CONFIG_INDEX] = BAUD_DETECT_ACCEPT_COUNT;     // TCR22
  TIR[CONFIG_DATA_PORT] = 0x00;

  TIR[CONFIG_INDEX] = BAUD_DETECT_FAIL_COUNT;       // TCR23
  TIR[CONFIG_DATA_PORT] = 0x00;

#endif  // COAX switch

  TIR[CONFIG_INDEX] = RSSI_SAMPLE_START;            // TCR24
  TIR[CONFIG_DATA_PORT] = SS(0);

  TIR[CONFIG_INDEX] = RSSI_CONFIGURATION;           // TCR25
  TIR[CONFIG_DATA_PORT] = A2DT(1);

  TIR[CONFIG_INDEX] = TIP_LED_SCRAMBLE;             // TCR27
  TIR[CONFIG_DATA_PORT] = 0x83;

  TIR[CONFIG_INDEX] = CCA_CONFIGURATION;            // TCR28
#ifdef COAX
  TIR[CONFIG_DATA_PORT] = UBDCS;
#else
  TIR[CONFIG_DATA_PORT] = ENXCHBSY;
#endif

  TIR[CONFIG_INDEX] = PIN_FUNCTION_DATA_RATE;       // TCR30
#ifdef COAX
  TIR[CONFIG_DATA_PORT] = 0x80 | DR(1);             // /ANTSLT pin driven
                                                    // Internal TXCLK
                                                    // 1Mbps (CLKGT20 = 1)
#else
  TIR[CONFIG_DATA_PORT] = 0x80 | TXCIN | DR(1);     // /ANTSLT pin driven
                                                    // External TXCLK
                                                    // 1Mbps (CLKGT20 = 1)
                                                    // Will rate switch to
                                                    // 2Mbps for QPSK frames
#endif


  TIR[CONFIG_INDEX] = PIN_CONFIGURATION_A;          // TCR13
  TIR[CONFIG_DATA_PORT] = 0xFF;

  TIR[CONFIG_INDEX] = PIN_CONFIGURATION_B;          // TCR14

#if defined (HARDWARE_DEBUG_PIN) && !defined (IMAX)
  TIR[CONFIG_DATA_PORT] = 0xD9;                     // Enables USER0 as debug output
#else
  TIR[CONFIG_DATA_PORT] = 0xD8;
#endif  // HARDWARE_DEBUG_PIN switch

  TIR[NETWORK_CONTROL] = LFPE;                      // TIR0
  TIR[NETWORK_CONTROL] = ACT | LNK;


#ifdef COAX
  // No need to initialize any PHY hardware
#else

/*
  The output pin LFPEJ is connected to the RESETJ input of the HSP3824.
  The writing of a 1 to the register bit LFPE outputs a 0 to the pin.
  In order to program the HSP3824 the resetj input must be false
*/

  if (initialize_3824() != 0) {
    /* initialize failed, return failure (FALSE) */
    return (FALSE);
  } /* end if */
  initialize_PLL();

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//* MKK regulatory domain specific routines

  if (reg_domain == 0x40)
    {      // MKK regulatory domain (Japan)
    Unsigned8 bytestream[20]=  {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,
                        0xB1,0x2B,0xA4,0xF9};
    // Make sure the TX DMA channel is stopped
    outportb(DMA_1_CONTROL, DEST_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
    disable_tx();
//    force_channel(12);        // Only DS channel in MKK domain for Alpha 1.5 and before
    force_channel(14);        // Only DS channel in MKK domain

    asm
      {
        push    cx
        mov    cx, 0ffffh  // ~50 millisecond pulse width
pk_loop:
        loop    pk_loop
        pop    cx
      }

    for (int i = 0; i < 8; i++)
      {
        TIR[TX_FIFO_DATA] = 0x55;                 // Preload TX FIFO with bit sync
        bytestream[12+i] = reg_domain_bytes[i];   // Fill bytestream with ID bytes
      }


    asm
      {
        push    cx
        mov    cx, 0ffffh  // ~50 millisecond pulse width
pk_kicker_loop:
        loop    pk_kicker_loop

        pop    cx
      }

    enable_tx();                   // Start TX state machine
    TIR[TXMT_CONTROL] = TXS;

    i = 0;
    while (i < 20)      //  Keep the FIFO full with the RCR33 data
      {
      while ((TIR[TXMT_STATUS]&0X01) && (i<20))
        {
          TIR[TX_FIFO_DATA] = bytestream[i];
          i++;
        } // while ((TIR[TXMT_STATUS]&0X01) && (i<20))
      }  // while (i < 20)
    }  //  if (reg_domain == 0x40)

#endif

  Control_Block.Rsvd_For_User_Rtn = 0;  // 3/7/97 RJT Clear user_function byte

  // Initialize local variables
  rel_time_correction_factor = 0;

  saved_RSSI_value = 0;

  return (TRUE);
} /* end initialize */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void pks_dummy ()
{
    asm
      {
        push    cx
        // Assert the debug pin (USER0) pin (LOW)
        mov    cx, 0ffffh  // ? millisecond pulse width
pk_loop:
        loop    pk_loop

        // Deassert the debug pin (USER0) pin (HIGH)
        pop    cx
        popf      // pop flags from the stack
      }
}


//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::preset_channel(const Unsigned8 channel)
{
  // This Radio API function is not called in DIRECT SEQUENCE implementations
} /* end preset_channel */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::change_channel(const Unsigned8 channel)
{
  // This Radio API function is not called in DIRECT SEQUENCE implementations
} /* end change_channel */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::force_channel(const Unsigned8 channel)
{
  asm {
    pushf      // push the flags on the stack
    cli      // disable interrupts
  }

#ifdef DEFAULT_DEBUG_CHANNEL
  const far char* ptr = (char far*) &astSynthesizer_RFNTable[DEFAULT_DEBUG_CHANNEL];
#else  // Use firmware controlled channel
  const far char* ptr = (char far*) &astSynthesizer_RFNTable[channel];
#endif  // DEFAULT_DEBUG_CHANNEL

  TIR[SERIAL_DEVICE] |= TIR2_SDC;    // SDC pin HIGH (Active LOW pulses)

  #ifdef IMAX
    // Assert /LE (/HFPE set LOW) to enable clock and data inputs
    TIR[NETWORK_CONTROL] |=HFPE;
  #else
    // Assert /LE (USER4) to enable clock and data inputs
    TIR[USER_PIN_DATA] &= ~TIR29_USERDT4;
  #endif

  TIR[FST_SER_PRT_CNTL] = ptr[0];
  TIR[FST_SER_PRT_CNTL] = ptr[1];
  TIR[FST_SER_PRT_CNTL] = ptr[2];
  TIR[FST_SER_PRT_CNTL] = ptr[3];
  TIR[FST_SER_PRT_CNTL] = ptr[4];

  #ifdef IMAX
    // Deassert /LE (HFPE set HIGH) to load shift register contents into synthesizer
    TIR[NETWORK_CONTROL] &=~HFPE;      //  Defined in *.hpp as 0x001
  #else
    // Deassert /LE (USER4) to load shift register contents into synthesizer
    TIR[USER_PIN_DATA] |= TIR29_USERDT4;
  #endif

  TIR[SERIAL_DEVICE] &= ~TIR2_SDC;    // SDC pin LOW

#ifdef COAX              // Used for coax channel separation simulation
  pre.set_SFD(channel);
                    // Alter the SFD in each transmitted frame
                    // to include the 'channel #'
                    // This makes frames 'unreceivable' unless
                    // receiver coax 'tuned' to correct channel


  TIR[CONFIG_INDEX] = START_DELIMITER_CSB;    // TCR9 (SFD detection logic)
  TIR[CONFIG_DATA_PORT] = channel;        // 'tunes' coax TX & RX
                                // (1st SFD byte transmitted)
#endif    // COAX channel simulation

  asm {
    push    cx
    // Assert the synthesizer 'kicker' (USER7) pin (LOW)
    or    byte ptr ds:[0400h+USER_PIN_DATA], TIR29_USERDT7L

    mov    cx, 1331  // 1 millisecond pulse width
  fc_kicker_loop:
    loop    fc_kicker_loop

    // Deassert the synthesizer 'kicker' (USER7) pin (HIGH)
    and    byte ptr ds:[0400h+USER_PIN_DATA], (NOT TIR29_USERDT7L) AND 0xFF

    pop    cx
    popf      // pop flags from the stack
  }

  /*  Turn off Activity LED since changing channel. */
//  TIR[NETWORK_CONTROL] &= ~ACT;

// DEBUG CODE TEMP. ONLY
//
//  TIR[NETWORK_CONTROL] ^= LNK;
//
// END DEBUG CODE

} /* end force_channel */




//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::wake()
{
//  Only valid for PCMCIA & Prism chipset
#ifndef IMAX

// Level 0 command execution time = ~8uS
// Level 1 command execution time = ~8uS
// Level 2 command execution time = ~19uS
//   Plus KICKER time if required

  //  Wake the Phy from the sleep mode.
  //  In reverse order.

  //  Debug use of USER 0
// TIR[USER_PIN_DATA] = TIR[USER_PIN_DATA] & 0xfe;          // TIR29
                  // Drives USER0 (debug pin) LOW (active)

  if (Sleep_Level >= 4)
  {
    //  Null for the time being
  } // if (level>=4)

  if (Sleep_Level >=3 )
  {
    //  Null for the time being
  } // if (level>=3)

  if (Sleep_Level >= 2)
  {
    // TIR0
    TIR[NETWORK_CONTROL] = Sleep_TIR0_mode2;

    // TIR11
    TIR[TXMT_SEQ_CNTL] = Sleep_TIR11_mode2;

    // TCR7
    TIR[CONFIG_INDEX] = PIN_DATA_A;
    TIR[CONFIG_DATA_PORT] = Sleep_TCR7_mode2;

    // TCR15
    TIR[CONFIG_INDEX] = PIN_CONFIGURATION_C;
    TIR[CONFIG_DATA_PORT] = Sleep_TCR15_mode2;

    //  TCR30
    TIR[CONFIG_INDEX] = PIN_FUNCTION_DATA_RATE;
    TIR[CONFIG_DATA_PORT] = Sleep_TCR30_mode2;
  }     // if (Sleep_Level >= 2)


     //  May need to be commented out for SUTROAPS.EXE support
  if (Sleep_Level >= 1)   //  Wake time required is ~1uS for Phy to stablize
                  //  and about 7uS to execute a wake from mode 1
  {
    TIR[NETWORK_CONTROL]=Sleep_TIR0_mode1;
  }     // if (Sleep_Level >= 1)

  //  Return LNK & STA LED's to pre-sleep mode.
  TIR[NETWORK_CONTROL]=Sleep_TIR0_mode0;

  //  The last thing to do befor returning control to the core code
/*   Core code asserts kicker
  if (Sleep_Level >= 3)
  {

      //  Kicker for Oscilator      Adds ~1mS to wake time
  asm
    {
    push    cx
    // Assert the synthesizer 'kicker' (USER7) pin (LOW)
    or    byte ptr ds:[0400h+USER_PIN_DATA], TIR29_USERDT7L

    mov    cx, 1331  // 1 millisecond pulse width
  pjk_kicker_loop:
    loop    pjk_kicker_loop

    // Deassert the synthesizer 'kicker' (USER7) pin (HIGH)
    and    byte ptr ds:[0400h+USER_PIN_DATA], (NOT TIR29_USERDT7L) AND 0xFF

    pop    cx
    popf      // pop flags from the stack
    }    // asm

  }     // if (Sleep_Level >= 3)
*/

  //  Debug use of USER 0
// TIR[USER_PIN_DATA] = TIR[USER_PIN_DATA] | 0x01;          // TIR29
                  // Drives USER0 (debug pin) HIGH (inactive)

#endif   //  #ifndef IMAX


} /* end wake */

void  Radio_API_Module::sleep(const Unsigned8 level)
{
//  Only valid for PCMCIA & Prism chipset
#ifndef IMAX

//  Level is the depth of the sleep that the Radio Phy will be placed in.
//  Each level coresponds to the modes as proposed by Harris for the Prism
//  chipset.
//  Level 0 ==  Sleep the PCNet Mobile Only
//  Level 1 ==  Mode 1 & Sleep the PCNet Mobile
//  Level 2 ==  Mode 2 & Sleep the PCNet Mobile
//  Level 3 ==  Mode 3 & Sleep the PCNet Mobile
//  Level 4 ==  Mode 4 & Sleep the PCNet Mobile
//  Each level is a superset of the prevous mode.
//  Thus a level 3 sleep first sleeps the Phy in Mode 1, then Mode 2 then Mode 3
//  then returns control to the core for 'sleeping' the PCNet Mobile.

  //  Debug use of USER 0
// TIR[USER_PIN_DATA] = TIR[USER_PIN_DATA] & 0xfe;          // TIR29
                  // Drives USER0 (debug pin) LOW (active)

// Level 0 command execution time = ~10uS
// Level 1 command execution time = ~19uS
// Level 2 command execution time = ~45uS

//  All sleep modes must first disable the RX & TX
//  Disable RX and TX
  /* stop transmitter and reset it */
  TIR[TXMT_CONTROL] = TXRES | TXFR;        // Transmit Reset must be
  TIR[TXMT_CONTROL] = 0;              // toggled on and off manually.

  /* reset receiver state machine */
  TIR[RCVR_CONTROL] = RXRES | RXFR;  // Reset SUTRO receiver
  TIR[NETWORK_CONTROL] |= RXP;    // Reset the HSP3824 receiver
  TIR[RCVR_CONTROL] = 0;          // Disable SUTRO reset.

  //  Turn off the LNK & ACT LED's to conserve power
  Sleep_TIR0_mode0 = TIR[NETWORK_CONTROL];  // Save value
  TIR[NETWORK_CONTROL]=Sleep_TIR0_mode0 & 0x3f;

  Sleep_Level = level;

  if (level >= 1)
  {
  //  Phy sleep mode 1 sets RXPE\ low
  Sleep_TIR0_mode1 = TIR[NETWORK_CONTROL];  // Save value
  TIR[NETWORK_CONTROL]=Sleep_TIR0_mode1 | 0x04;
  }  // if (level >= 1)

  if (level>=2)
  {
  //  Save current register states
  Sleep_TIR0_mode2  = TIR[NETWORK_CONTROL];

  Sleep_TIR11_mode2 = TIR[TXMT_SEQ_CNTL];

  TIR[CONFIG_INDEX] = PIN_DATA_A;
  Sleep_TCR7_mode2  = TIR[CONFIG_DATA_PORT];

  TIR[CONFIG_INDEX] = PIN_CONFIGURATION_C;
  Sleep_TCR15_mode2 = TIR[CONFIG_DATA_PORT];

  TIR[CONFIG_INDEX] = PIN_FUNCTION_DATA_RATE;
  Sleep_TCR30_mode2 = TIR[CONFIG_DATA_PORT];

  //  Set new values

  // TIR0
  TIR[NETWORK_CONTROL] = Sleep_TIR0_mode2 | 0x02;

  // TIR11
  TIR[TXMT_SEQ_CNTL] = Sleep_TIR11_mode2 | 0x09;

  // TCR7
  TIR[CONFIG_INDEX] = PIN_DATA_A;
  TIR[CONFIG_DATA_PORT] = Sleep_TCR7_mode2 & 0xfb;

  // TCR15
  TIR[CONFIG_INDEX] = PIN_CONFIGURATION_C;
  TIR[CONFIG_DATA_PORT] = Sleep_TCR15_mode2 | 0x20;

  //  TCR30
  TIR[CONFIG_INDEX] = PIN_FUNCTION_DATA_RATE;
  TIR[CONFIG_DATA_PORT] = Sleep_TCR30_mode2 | 0x20;

  } // if (level>=2)

  if (level >=3 )
  {
  //  Null for the time being
  } // if (level>=3)

  if (level >=4 )
  {
  //  Null for the time being
  } // if (level>=4)

  //  Debug use of USER 0
// TIR[USER_PIN_DATA] = TIR[USER_PIN_DATA]| 0x01;          // TIR29
                  // Drives USER0 (debug pin) HIGH (inactive)

#endif   //  #ifndef IMAX


} /* end sleep */




//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::arm_tx()
{
  TIR[TXMT_CONTROL] = Next_TX_Cntl;
} // end arm_tx

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::enable_tx()
{
  // Hold the HSP3824 receiver in reset
  TIR[NETWORK_CONTROL] |= RXP;

  TIR[RCVR_CONTROL] = RXRES | RXFR;        // Receive Reset must be toggled on
  TIR[RCVR_CONTROL] = 0;              // and off manually.

  // start transmitter
#  ifdef COAX
    TIR[TXMT_CONTROL] = TXS | CRC32;
                          // Enable to start a transmission.
                          // Do not use dynamic rate switch
                          // for COAX builds
#  else
#    ifdef FORCE_QPSK
      TIR[TXMT_CONTROL] = TXS | CRC32 | TIR8_DR;
                          // Enable to start a transmission.
                          // Uses dynamic rate switch
#    else
      TIR[TXMT_CONTROL] = Next_TX_Cntl & ~TIR8_ENCRCTX;
//      TIR[TXMT_CONTROL] = TXS | CRC32;
                          // Enable to start a transmission.
                          // Does not use dynamic rate switch
#    endif
#  endif

} /* end enable_tx */





//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//const Unsigned8 Radio_API_Module::enable_response_if_good(const Unsigned16 good_length) const
///*
// *  This routine is called from PHY_Rx for a received frame that needs a
// *  response. In order to meet the SIFS time the transmitter is autostarted
// *  at the response rate set in get_PLCP info. This routine will wait until
// *  a correct length with the CRC good is received, or a length >= good_length
// *  with a bad CRC is received or a timeout occurs.  Good CRC, Bad CRC or
// *  timeout status is  returned.
// */
//{
//  /* Start transmitter for response TX even if already started */
//#  ifdef COAX
//    TIR[TXMT_CONTROL] = TXS | CRC32 | TIR8_ENCRCTX;
//                      // Enable to start a transmission.
//                      // Do not use dynamic rate switch
//                      // for COAX builds
//#  else
//#    ifdef FORCE_QPSK
//      TIR[TXMT_CONTROL] = TXS | CRC32 | TIR8_ENCRCTX | TIR8_DR;
//                      // Enable to start a transmission.
//                      // Uses dynamic rate switch
//#    else
//      TIR[TXMT_CONTROL] = Next_Response_TX_Cntl;
//                      // Enable to start a transmission.
//                      // Uses dynamic rate switch
//#    endif
//#  endif
//
//
//  int i = RX_DMA_TIMEOUT_VALUE;
//  register Boolean got_length = FALSE;
//  do {
//    if (*((volatile Unsigned16* const)&(TIR[CRC32_CORRECT_LSB])) == good_length)
//    {
//      /* stop_rx_DMA() */
//      outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//      if ((TIR[TXMT_STATUS] & TXBSY) != 0) {
//        /* transmitter started, return successfull */
//        TIR[NETWORK_CONTROL] |= RXP;  // Hold the HSP3824 RX in reset
//                        // SUTRO RX state machine still on
//                        // (TX in progress)
//        TIR[RCVR_CONTROL] = RXFR;    // Hold RX FIFO in reset
//                        // RX DMA will be started soon,
//                        // must have empty RX FIFO
//        return(GOOD_CRC);
//      } else {
//        /* transmitter failed to start, treat as bad CRC */
//        TIR[TXMT_CONTROL] = TXRES;    // Reset (auto) Transmitter
//        TIR[TXMT_CONTROL] = 0;      // (RX frame never arrived)
//        return(BAD_CRC);
//      } /* end if */
//    } /* end if */
//
//    if ((*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) >= good_length) ||
//      (*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) == 0))
//    {
//      if (got_length != FALSE) {
//        /* stop_rx_DMA() */
//        outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//        /* got byte count == 0 or >= good_length more than once */
//        // Expected CRC32 failed
//        TIR[TXMT_CONTROL] = TXRES;    // Reset (auto) Transmitter
//        TIR[TXMT_CONTROL] = 0;      // (RX frame never arrived)
//        return(BAD_CRC);
//      } else {
//        /* got byte count == 0 or >= good_length once */
//        got_length = TRUE;
//      } /* end if */
//    } else {
//      got_length = FALSE;
//    } /* end if */
//  } while (--i > 0); /* end do */
//
//  /* stop_rx_DMA() */
//  outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//  /* wait loop timed out */
//  TIR[TXMT_CONTROL] = TXRES;          // Reset (auto) Transmitter
//  TIR[TXMT_CONTROL] = 0;            // (RX frame never arrived)
//  return(RX_DMA_TIMEOUT);
//
//} /* end enable_response_if_good */
//
//
//
//const Unsigned8 Radio_API_Module::enable_tx_if_good(const Unsigned16 good_length) const
///*
// *  This routine is called from PHY_Rx for a received frame that needs a
// *  next frame to be sent. In order to meet the SIFS time the transmitter
// *  is autostarted at the rate previously set in the set_PHY_rate call.
// *  This routine will wait until a correct length with the CRC good is
// *  received, or a length >= good_length with a bad CRC is received or a
// *  timeout occurs.  Good CRC, Bad CRC or timeout status is  returned.
// */
//{
//  /* Start transmitter for TX even if already started */
//#  ifdef COAX
//    TIR[TXMT_CONTROL] = TXS | CRC32 | TIR8_ENCRCTX;
//                      // Enable to start a transmission.
//                      // Do not use dynamic rate switch
//                      // for COAX builds
//#  else
//#    ifdef FORCE_QPSK
//      TIR[TXMT_CONTROL] = TXS | CRC32 | TIR8_ENCRCTX | TIR8_DR;
//                      // Enable to start a transmission.
//                      // Uses dynamic rate switch
//#    else
//      TIR[TXMT_CONTROL] = Next_TX_Cntl;
//                      // Enable to start a transmission.
//                      // Uses dynamic rate switch
//#    endif
//#  endif
//
//
//  int i = RX_DMA_TIMEOUT_VALUE;
//  register Boolean got_length = FALSE;
//  do {
//    if (*((volatile Unsigned16* const)&(TIR[CRC32_CORRECT_LSB])) == good_length)
//    {
//      /* stop_rx_DMA() */
//      outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//      if ((TIR[TXMT_STATUS] & TXBSY) != 0) {
//        /* transmitter started, return successfull */
//        TIR[NETWORK_CONTROL] |= RXP;  // Hold the HSP3824 RX in reset
//                        // SUTRO RX state machine still on
//                        // (TX in progress)
//        TIR[RCVR_CONTROL] = RXFR;    // Hold RX FIFO in reset
//                        // RX DMA will be started soon,
//                        // must have empty RX FIFO
//        return(GOOD_CRC);
//      } else {
//        /* transmitter failed to start, treat as bad CRC */
//        TIR[TXMT_CONTROL] = TXRES;    // Reset (auto) Transmitter
//        TIR[TXMT_CONTROL] = 0;      // (RX frame never arrived)
//        return(BAD_CRC);
//      } /* end if */
//    } /* end if */
//
//    if ((*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) >= good_length) ||
//      (*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) == 0))
//    {
//      if (got_length != FALSE) {
//        /* stop_rx_DMA() */
//        outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//        /* got byte count == 0 or >= good_length more than once */
//        // Expected CRC32 failed
//        TIR[TXMT_CONTROL] = TXRES;    // Reset (auto) Transmitter
//        TIR[TXMT_CONTROL] = 0;      // (RX frame never arrived)
//        return(BAD_CRC);
//      } else {
//        /* got byte count == 0 or >= good_length once */
//        got_length = TRUE;
//      } /* end if */
//    } else {
//      got_length = FALSE;
//    } /* end if */
//  } while (--i > 0); /* end do */
//
//  /* stop_rx_DMA() */
//  outportb(DMA_0_CONTROL, SRC_SYNC | LO_PRIORITY | TDRQ_DIS | CHANGE | DMA_STOP);
//
//  /* wait loop timed out */
//  TIR[TXMT_CONTROL] = TXRES;          // Reset (auto) Transmitter
//  TIR[TXMT_CONTROL] = 0;            // (RX frame never arrived)
//  return(RX_DMA_TIMEOUT);
//
//} /* end enable_tx_if_good */
//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^



//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::stop_tx()
{
  /* stop transmitter and reset it */
  TIR[TXMT_CONTROL] = TXRES;            // Transmit Reset must be
  TIR[TXMT_CONTROL] = 0;              // toggled on and off manually.

} /* end stop_tx */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::disable_tx()
{
  /* stop transmitter and reset it */
  TIR[TXMT_CONTROL] = TXRES | TXFR;        // Transmit Reset must be
  TIR[TXMT_CONTROL] = 0;              // toggled on and off manually.

  /* reset and enable receiver */
  TIR[RCVR_CONTROL] = RXRES | RXFR;        // Reset SUTRO receiver
//AS  TIR[NETWORK_CONTROL] |= RXP;          // Reset the HSP3824 receiver
  TIR[RCVR_CONTROL] = 0;              // Disable SUTRO reset.
  TIR[RCVR_CONTROL] = RXS;            // Start SUTRO receiver again.

  /* enable the HSP3824 receiver */
  TIR[NETWORK_CONTROL] &= ~(RXP);          // Enable the HSP3824 receiver

} /* end disable_tx */




//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::reset_CCA()
/*
 *  This call will contain any code specific to a radio that does its own CCA
 *  and needs a signal to reset (restart) the CCA state machine.  This signal
 *  is called each time after receiver is started, and when media goes idle.
 *
 *  In the pulse radio case antenna diversity must be actively switching in
 *  order for frames to be detected.  Diversity may have stopped because of a
 *  false baud detect, with no SFD interrupt following it.  Since carrier sense
 *  will always occur if a Stop Diversity condition happens the transition back
 *  to carrier IDLE will be used to also re-start diversity by writing the RXS
 *  bit.
 */
{
  // *****RJT**RE EXAMINE THIS*****
  register Unsigned8 rcvr_control;

  rcvr_control = TIR[RCVR_CONTROL];   // Save the current SUTRO RX state

  /* reset receiver state machines */
  TIR[NETWORK_CONTROL] |= RXP;    // Reset the HSP3824 receiver  //TT

  TIR[RCVR_CONTROL] = RXRES | RXFR;  // Reset SUTRO receiver
//TT  TIR[NETWORK_CONTROL] |= RXP;    // Reset the HSP3824 receiver
  TIR[RCVR_CONTROL] = 0;          // Disable SUTRO reset.
  TIR[RCVR_CONTROL] = rcvr_control;   // Restore SUTRO RX state

  // *****RJT**HSP3824 CODE?*****
  TIR[NETWORK_CONTROL] &= ~(RXP);    // Enable the HSP3824 receiver

  /*  Turn activity LED off.  */
  //TIR[NETWORK_CONTROL] &= ~ACT;
} /* end reset_CCA */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::enable_rx()
{
  /*  turn activity LED on. */
  //TIR[NETWORK_CONTROL] |= ACT;

  /* stop transmitter (if running) by resetting TX state machine only */
  /* Do NOT reset the TX FIFO */
  TIR[TXMT_CONTROL] = TXRES;          // Transmit Reset must be
  TIR[TXMT_CONTROL] = 0;            // toggled on and off manually.

  /* reset receiver state machine */
  TIR[NETWORK_CONTROL] |= RXP;    // Reset the HSP3824 receiver //TT

  TIR[RCVR_CONTROL] = RXRES | RXFR;  // Reset SUTRO receiver
//TT  TIR[NETWORK_CONTROL] |= RXP;    // Reset the HSP3824 receiver
  TIR[RCVR_CONTROL] = 0;          // Disable SUTRO reset.

  // start the SUTRO receive state machine
  TIR[RCVR_CONTROL] = RXS;        // Start SUTRO receiver

  /* enable the HSP3824 receiver */
  TIR[NETWORK_CONTROL] &= ~(RXP);    // Enable the HSP3824 receiver
} /* end enable_rx */


//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//const Unsigned32 Radio_API_Module::get_PLCP_info(const Unsigned8* PLCP_ptr) const
//{
//  if ((((PLCP_Header_Type*)PLCP_ptr)->get_rate()) == TWO_MBIT) {
//    Next_Response_TX_Cntl = TXS | CRC32 | TIR8_ENCRCTX | TIR8_DR;
//  } else {
//    Next_Response_TX_Cntl = TXS | CRC32 | TIR8_ENCRCTX | 0;
//  } /* end if */
//
//  if (*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) < 6) {
//
//    int delay = 32;
//
//    while (*((volatile Unsigned16* const)&TIR[BYTE_COUNT_LSB]) < 6) {
//
//      if (--delay <= 0) {
//
//        return (0x80000000L);
//      } /* end if */
//    } /* end while */
//  } /* end if */
//
//  return ((Unsigned32(((PLCP_Header_Type*)PLCP_ptr)->get_length()) << 16) |
//    ((PLCP_Header_Type*)PLCP_ptr)->get_rate());
//} /* end get_PLCP_info */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned8 Radio_API_Module::get_rx_PLCP_header_len() const
{
  return(PLCP_LEN);
} /* end get_rx_PLCP_header_len */


//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const  Unsigned8  Radio_API_Module::get_RSSI() const
{
  // *****RJT**NEED TO REWRITE THIS*****
  /*  turn activity LED on. */
//  TIR[NETWORK_CONTROL] |= ACT;
  return(saved_RSSI_value);
} /* end get_RSSI */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned16 Radio_API_Module::get_length(const Unsigned8* PLCP_ptr)const
{
  return(((PLCP_Header_Type*)PLCP_ptr)->get_length());

} /* end get_length */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Boolean Radio_API_Module::is_PLCP_header_good(const Unsigned8* PLCP_ptr)const
{
  if (((PLCP_Header_Type*)PLCP_ptr)->get_EC() == ((PLCP_Header_Type*)PLCP_ptr)->CRC_16()) {
    return (TRUE);
  }
  else {
    return(FALSE);
  }

} /* end is_PLCP_header_good */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::build_PLCP_header(const Unsigned8* PLCP_start,
                const Unsigned16 length, const Unsigned8 rate)
{
  ((PLCP_Header_Type*)PLCP_start)->set_signal(rate);
  ((PLCP_Header_Type*)PLCP_start)->set_service(rate);
  if (rate == ONE_MBIT) {
    ((PLCP_Header_Type*)PLCP_start)->set_length1Mb(length);
  }
  else {
    ((PLCP_Header_Type*)PLCP_start)->set_length2Mb(length);
  }
  ((PLCP_Header_Type*)PLCP_start)->set_EC();

} /* end build_PCLP_header */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned8 Radio_API_Module::get_tx_PLCP_header_len() const
{
  return(PLCP_LEN);

} /* end get_tx_PLCP_header_size */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
Unsigned8* Radio_API_Module::get_tx_preamble() const
{
  return(pre.preamble);

} /* end get_tx_preamble */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned8 Radio_API_Module::get_tx_preamble_len() const
{
  return(PREAMBLE_LEN + SFD_LEN);

} /* end get_tx_preamble_len */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned8 Radio_API_Module::get_PHY_type() const
{
  return(DIRECT_SEQUENCE);
} /* end get_PHY_type */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned8 Radio_API_Module::get_hdw_delay() const
{
  return(0);
} /* end get_hdw_delay */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void Radio_API_Module::set_PHY_rate(const Unsigned8 rate)
{

#  ifdef FORCE_QPSK
    Next_TX_Cntl = TXS | CRC32 | TIR8_ENCRCTX | TIR8_DR;  // FORCE QPSK
#  else
    if (rate == TWO_MBIT) {
      Next_TX_Cntl = TXS | CRC32 | TIR8_ENCRCTX | TIR8_DR;
    } else {
      Next_TX_Cntl = TXS | CRC32 | TIR8_ENCRCTX;
    } /* end if */
#  endif /* end FORCE_QPSK */
                  // Software flag only; used to select
                  // TX PHY rate in enable_tx() and
                  // enable_tx_if_good().
                  // Hardware PHY rate selection is done
                  // manually for TX only. This is done in
                  // enable_tx() and enable_tx_if_good().
                  // Hardware PHY rate selection for RX
                  // is done automatically by the
                  // HSP3824 chip.

} /* end set_PHY_rate */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned16 Radio_API_Module::rel_time_to_usec_est(
                          const Unsigned16 clk_ticks)
/*
 *  The free running clk is assumed to be 32768 Hz in this routine. This
 *  gives a timer tick time of 30.517 usec. This routine estimates that as
 *  30.5.
 *
 *  This routine only works for differences where the result is less than the
 *  maximum value that can fit into a 16-bit value (~65 msec).
 */
{
  return Unsigned16((clk_ticks*32) - clk_ticks - (clk_ticks/2));  // 32768 Hz

} /* end rel_time_to_usec_est */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned32 Radio_API_Module::rel_time_to_usec(
                          const Unsigned24& rel_time)
/*
 *  This routine converts a 24-bit number (max allowed value 0x7fffff) of
 * real time clock (usually 32767 Hz) ticks to usec. The conversion
 * is accurate to 1 usec. Partial usec  are truncated since the error is not
 * expected to accumulate.
 *
 *  This routine uses 32-bit multiplies and shifts and will be time consuming.
 *  A faster, less accurate conversion is available for values where the result
 *  will fit into a 16-bit word.
 *
 *  It is assumed that rel_time_to_usec is only called by Update_Time in
 *  Sys_Timer so that the rel_time_correction_factor can be used and updated
 *  on each call.
 */
{
  Unsigned32 low_portion =
    ((rel_time & 0x1ffL) * (USEC_PER_SEC*0x200L/FREE_RUN_CLK_HZ));

  Unsigned32 result =
    ((rel_time/0x200L) * (USEC_PER_SEC*0x200L/FREE_RUN_CLK_HZ)) +
    ((low_portion + rel_time_correction_factor)/0x200L);

  rel_time_correction_factor =
    (((Unsigned16)low_portion + rel_time_correction_factor) & 0x1ff);

  return (result);

} /* end rel_time_to_usec */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
const Unsigned32 Radio_API_Module::usec_to_rel_time(
                          const Unsigned32& usec_time)
/*
 *  This routine converts a 32 bit number (max allowed value 8,000,000) in units
 *  of usecs. to 32768 Hz clk ticks. The conversion is accurate to 1 tick.
 *  Partial ticks are truncated since the error is not expected to accumulate.
 *
 *  This routine accomplishes the divide by 30.51 usec by first multiplying
 *  by 512 and dividing by 15625. (equiv. to 32768/1000000)
 */
{

  return Unsigned32((usec_time * 0x200L) /
            (USEC_PER_SEC*0x200L/FREE_RUN_CLK_HZ));

} /* end usec_to_rel_time */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::pgm_clkgt20() const
/*
 *  Set CLKGT20 bit since CPU is operating at 20 MHz (40 MHz system clk).
 */
{
  MIR[MISC] |= CLKGT20;

} /* end pgm_clkgt20 */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::set_wait_states() const
{
//    Wait state settings for various memory speeds are as follows:
//
//    MEMORY SPEED                        WAIT STATES
//    ********************************************    ***********
//  tACC 55ns MAX ,tCE 55ns MAX ,tOE 30ns MAX          0
//  tACC 105ns MAX ,tCE 105ns MAX ,tOE 80ns MAX        1
//  tACC 155ns MAX ,tCE 155ns MAX ,tOE 130ns MAX        2

// *  Set both Flash and SRAM to 1 wait state for 20 MHz CPU clock.
//
//  MIR[FLASH_WAIT] = (MIR[FLASH_WAIT] & ~FLASHWAIT3) | FLASHWAIT1;
//  MIR[MISC] = (MIR[MISC] & ~SRAMWAIT3) | SRAMWAIT1;

//
// *  Set both Flash and SRAM to 0 wait states for 20 MHz CPU clock.
//
  MIR[FLASH_WAIT] = (MIR[FLASH_WAIT] & ~FLASHWAIT3) | FLASHWAIT0;
  MIR[MISC] = (MIR[MISC] & ~SRAMWAIT3) | SRAMWAIT0;

} /* end set_wait_states */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void  Radio_API_Module::user_function()
{
/*
 ** Setting this following definition:
 ** TRUE will enable compilation of the test mode mode routines.
**  FALSE will enable the compilation of the "normal" mode routines.
*/

#define BLD_TEST_MODE_RADIO_API       FALSE
//#define BLD_TEST_MODE_RADIO_API       TRUE


//***************************************************************************
//*
//* THIS SECTION OF CODE IS INCLUDED FOR MANUFACTURER SPECIFIC LOGIC.  THIS
//* ROUTINE IS EXECUTED ONCE EVERY BACKGROUND SYSTEM LOOP.
//* THE CURRENT SAMPLE CAN BE USED FOR TESTING SPECIFIC FUNCTIONS OR
//* PROVIDING REGULATORY SPECIFIC ROUTINES.
//*
//* PLEASE REFER THE THE PCNET MOBILE 802.11 AP/STA DRIVER INTERFACE
//* SPECIFICATION FOR FURTHER DETAILS AS TO THE USE OF THIS FUNCTION.
//*
//* This routine is a sample user function which allows the manufacturer to
//* test radio functions.  They are here as an example.  The manufacture
//* is encouraged to modify this to apply to their specific needs.
//*
//***************************************************************************

#if BLD_TEST_MODE_RADIO_API ==  TRUE

  switch(Control_Block.Rsvd_For_User_Rtn) {
    case 0:                    // Null command (should never be called)
      break;
    case 1:                    // Tune to channel 1
      force_channel(1);
      break;
    case 2:                    // Tune to channel 2
      force_channel(2);
      break;
    case 3:                    // Tune to channel 3
      force_channel(3);
      break;
    case 4:                    // Tune to channel 4
      force_channel(4);
      break;
    case 5:                    // Tune to channel 5
      force_channel(5);
      break;
    case 6:                    // Tune to channel 6
      force_channel(6);
      break;
    case 7:                    // Tune to channel 7
      force_channel(7);
      break;
    case 8:                    // Tune to channel 8
      force_channel(8);
      break;
    case 9:                    // Tune to channel 9
      force_channel(9);
      break;
    case 10:                    // Tune to channel 10
      force_channel(10);
      break;
    case 11:                    // Tune to channel 11
      force_channel(11);
      break;
    case 12:                    // Tune to channel 12
      force_channel(12);
      break;
    case 13:                    // Tune to channel 13
      force_channel(13);
      break;
    case 14:                    // Tune to channel 14
      force_channel(14);
      break;
    case 15:                    // Toggle ACT LED
      TIR[NETWORK_CONTROL] ^= ACT;
      break;
    case 16:                    // Toggle LNK LED
      TIR[NETWORK_CONTROL] ^= LNK;
      break;
    case 17:
//* Undefined...
      break ;

    case 18:                    // Continuous PN15 transmission

//* This is a user test function which is used to assist the customer in
//* tuning there transmitters.  Once this function has been invoke, it will
//* require a user to reset the system in order to return to normal
//* protocol operation.

                            // Once invoked, adapter must be
                            // reset to exit this user function

      // ***************************************************************
      // ***** THIS TEST ROUTINE RUDELY TAKES OVER ADAPTER MEMORY  *****
      // ***** THE ADAPTER MUST BE RESET TO EXIT THIS TEST ROUTINE *****
      // ***************************************************************

      Unsigned16 i, pn_15;
      Unsigned8 far *pn_ptr, far *xor_table;
      asm {
        cli
      }
      /* stop transmitter and reset it */
      TIR[TXMT_CONTROL] = TXRES | TXFR;   // Reset TX state machine
      TIR[NETWORK_CONTROL] |= RXP ;         //      DEASSERT_3824_RXPE                  // Disable the HSP3824 receiver
      pn_ptr =(Unsigned8*)MK_FP(0,0x500);
      xor_table =(Unsigned8*)MK_FP(0,0x1500);
      for (i=0;i<128;i++) {               // Build the PN_15 sequence
        xor_table[i]  = ((i&0x01)?0x7F:0) \
            ^ ((i&0x02)?0x7E:0) \
                  ^ ((i&0x04)?0x7C:0) \
                  ^ ((i&0x08)?0x78:0) \
                  ^ ((i&0x10)?0x70:0) \
                  ^ ((i&0x20)?0x60:0) \
                  ^ ((i&0x40)?0x40:0);
      }
      pn_15 = 0x7FFF;
      for (i=0;i<0x1000;i++) {            // Build the PN_15 sequence
        pn_ptr[i] = (Unsigned8)(pn_15&0xFF);
        pn_15 = ((pn_15>>8)&0x7F) \
            ^ ((pn_15&0x4000)?0x7F80:0) \
            ^ ((pn_15&1)?0x7F80:0) \
            ^ (((Unsigned16)(xor_table[(pn_15>>1)&0x7F]))<<8);
      }
      TIR[TXMT_CONTROL] = 0;              // Unreset the TX state machine
      TIR[TX_FIFO_DATA] = 0xFF;           // Preload the TX FIFO with
      TIR[TX_FIFO_DATA] = 0x7F;           //  first 8 bytes from PN15
      TIR[TX_FIFO_DATA] = 0x55;           //  sequence
      TIR[TX_FIFO_DATA] = 0x95;           //
      TIR[TX_FIFO_DATA] = 0x99;           //
      TIR[TX_FIFO_DATA] = 0xB9;           //
      TIR[TX_FIFO_DATA] = 0xBB;           //
      TIR[TX_FIFO_DATA] = 0x4B;           //
      TIR[NETWORK_CONTROL] |= ACT | LNK;  // Turn on LNK and ACT LEDs

      asm {
        xor ax, ax
        mov ds, ax
        mov si, 0x508                     // Pointer to 9th PN15 value
        mov cx, 0xFF8                     // 4K - (8 FIFO preloads)
        mov al, [si]                      // 9th PN15 value
        mov byte ptr ds:[400h+TXMT_CONTROL], TXS    // Start TX @ 1Mbps
        wait_for_fifo:
        test  byte ptr ds:[400h+NETWORK_STATUS], 1    // TX FIFO DRQ?
        jz    test_for_uflo
        write_byte:
        mov ds:[400h+TX_FIFO_DATA], al
        next_byte:
        inc si
        mov al, [si]
        loop  wait_for_fifo
        sequence_start:
        mov si, 0x500
        mov al, [si]                      // First PN15 value
        mov cx, 0x1000
        xor byte ptr ds:[400h+NETWORK_CONTROL], LNK // Flicker LNK LED
        jmp   wait_for_fifo
        test_for_uflo:
        test  byte ptr ds:[400h+TXMT_STATUS], 1     // TX still busy?
        jnz wait_for_fifo                           // Good, keep TXing
        tx_uflo:
        mov   byte ptr ds:[0400h+NETWORK_CONTROL], LNK OR ACT
                                          //ACT and LNK LEDs
                                          //..will flash in unison

        we_are_hung:                      // Infinite loop label
        mov   cx, 25                      // 25 outer loops
                                          //..(10 milliseconds per loop)
        outer_fail_loop:
        push    cx
        mov   cx, 13310                   // 10 millisecond loop delay
        inner_fail_loop:
        loop    inner_fail_loop
        pop   cx
        loop    outer_fail_loop
        xor   byte ptr ds:[0400h+NETWORK_CONTROL], LNK OR ACT
                                          // Toggle LNK and ACT LEDs
                                          //..until adapter is reset
        jmp   we_are_hung                 // Infinite loop
      }
      // Will never get here (test routine has no exit)
    default:                    // Unknown user function
      break;
   } // End switch

  Control_Block.Rsvd_For_User_Rtn = 0;  // Clear current user_function byte


/* endif BLD_TEST_MODE_RADIO_API ==  TRUE */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
#else /* BLD_TEST_MODE_RADIO_API !=  TRUE */
//***************************************************************************
//*
//* THIS SECTION OF CODE IS INCLUDED FOR MANUFACTURER SPECIFIC LOGIC.  THIS
//* ROUTINE IS EXECUTED ONCE EVERY BACKGROUND SYSTEM LOOP.
//* THE CURRENT SAMPLE CAN BE USED FOR DRIVER/SYSTEM SPECIFIC FUNCTIONS OR
//* PROVIDING REGULATORY SPECIFIC ROUTINES.
//*
//* PLEASE REFER THE THE PCNET MOBILE 802.11 AP/STA DRIVER INTERFACE
//* SPECIFICATION FOR FURTHER DETAILS AS TO THE USE OF THIS FUNCTION.
//*
//* This routine is a sample user function which allows the manufacturer to
//* use the ACTIVITY and LINK LEDs to indicate system functions.  They are
//* here as an example.
//*
//* The manufacture is encouraged to modify this to apply to their specific
//* needs.
//*
//***************************************************************************

  Unsigned8 user_byte;

  user_byte = Control_Block.Driver_State ;

  if (user_byte & SCAN_BIT)
  {
    if (MIR[FREE_CNT_M] & 0x10)        // Fast flash rate on LNK led.
      TIR[NETWORK_CONTROL] |= LNK;
    else
      TIR[NETWORK_CONTROL] &= ~LNK;
  }
  else if (user_byte & AP_BIT)      // Slow or Medium flash on LNK Led
  {                                 // depending on value in State bit.
    if ( ((MIR[FREE_CNT_M] & 0x80) && !(user_byte & STATE_BIT))  ||
       ((MIR[FREE_CNT_M] & 0x20) && (user_byte & STATE_BIT))  )
      TIR[NETWORK_CONTROL] |= LNK;
    else
      TIR[NETWORK_CONTROL] &= ~LNK;
  }
  else if (user_byte & MODE_BIT)
  {
    if (user_byte & STATE_BIT)
      TIR[NETWORK_CONTROL] |= LNK;
    else
      TIR[NETWORK_CONTROL] &= ~LNK;
  }
  else if (user_byte & STATE_BIT)
  {                                     // Blip LED every second.
    if (MIR[FREE_CNT_M] & 0x70)
      TIR[NETWORK_CONTROL] &= ~LNK;
    else
      TIR[NETWORK_CONTROL] |= LNK;
  }
  else
      TIR[NETWORK_CONTROL]&= ~LNK;

//    if (Control_Block.Rsvd_For_User_Rtn & NETWORK_MODE_BIT)
//      TIR[NETWORK_CONTROL] |= LNK;
//      TIR[NETWORK_CONTROL] &= ~LNK;
//    else
//    {
//      if ((char )MIR[FREE_CNT_M] < 0)
//        TIR[NETWORK_CONTROL] |= LNK;
//        TIR[NETWORK_CONTROL] &= ~LNK;
//      else
//        TIR[NETWORK_CONTROL] &= ~LNK;
//        TIR[NETWORK_CONTROL] |= LNK;
//    }
//  }
//  else
//  {
//    TIR[NETWORK_CONTROL] &= ~ACT;
//    TIR[NETWORK_CONTROL] &= ~LNK;
//    TIR[NETWORK_CONTROL] |= LNK;
//  }

#endif /* BLD_TEST_MODE_RADIO_API !=  TRUE */

} /* end user_function */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
// The following function performs writes and verifications of HSP3824
// CR registers as defined in the constant table astHSP3824_Init_Table[].
//
//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
Unsigned8 initialize_3824(void) {

#define ATTEMPTS_PER_REGISTER 2      // # of attempts to initialize
#define WRITEABLE_REGISTERS  (sizeof astH3824_Init_Table) / kcbH3824RegWriteTableElement

#define kBlockPrgmAttempts  4

  Unsigned8 return_value;
  Unsigned8 write_reg, attempts_left;
  const far char* ptr;

  const h3824_data_struct astH3824_Init_Table[] = {
    {{ H3824WElmt( 0,  0x1C ) }} ,    // MODEM config A  (per Harris 10/8/96)
//    {{ H3824WElmt( 0,  0x18 ) }} ,    // Old MODEM config A
    {{ H3824WElmt( 1,  0x02 ) }} ,    // MODEM config B
    {{ H3824WElmt( 2,  0x27 ) }} ,    // MODEM config C
    {{ H3824WElmt( 3,  0x00 ) }} ,    // MODEM config D
    {{ H3824WElmt( 4,  0x00 ) }} ,    // Test Register (0x00)
//    {{ H3824WElmt( 5,  0x40 ) }} ,    // Mfg. Test (0x40 disables clock jump)
/* Per Harris 3/25/98 */
    {{ H3824WElmt( 5,  0xc0 ) }} ,    // Mfg. Test (0x40 disables clock jump)
                                      // ..invert receive Q on HFA3824A
//    {{ H3824WElmt( 6,  0x00 ) }} ,    //
//    {{ H3824WElmt( 7,  0x00 ) }} ,    // MODEM Status A
//    {{ H3824WElmt( 8,  0x00 ) }} ,    // MODEM Status B
    {{ H3824WElmt( 9,  0x00 ) }} ,    // I/O Definition Register
//    {{ H3824WElmt( 10, 0x00 ) }} ,    // RSSI Value Register
    {{ H3824WElmt( 11, 0x01 ) }} ,    // A/D Cal Positive Register
    {{ H3824WElmt( 12, 0xFD ) }} ,    // A/D Cal Negative Register
    {{ H3824WElmt( 13, 0x05 ) }} ,    // TX spread code (high)
    {{ H3824WElmt( 14, 0xB8 ) }} ,    // TX spread code (low)
    {{ H3824WElmt( 15, 0x70 ) }} ,    // TX scrambler seed
//    {{ H3824WElmt( 16, 0x48 ) }} ,    // TX scrambler taps
/* Per Harris 3/25/98 */
    {{ H3824WElmt( 16, 0xc8 ) }} ,    // TX scrambler taps
                                      // ..invert transmit Q on HFA3824A
    {{ H3824WElmt( 17, 0x2C ) }} ,    // CCA timer threshold
    {{ H3824WElmt( 18, 0x03 ) }} ,    // CCA cycle threshold
    {{ H3824WElmt( 19, 0x3F ) }} ,    // RSSI threshold, energy detect
    {{ H3824WElmt( 20, 0x05 ) }} ,    // RX spread code (high)
    {{ H3824WElmt( 21, 0xB8 ) }} ,    // RX spread code (low)
//    {{ H3824WElmt( 22, 0x01 ) }} ,    // Old RX SQ1 ACQ Threshold (high)
//    {{ H3824WElmt( 23, 0xE8 ) }} ,    // Old RX SQ1 ACQ Threshold (low)
    {{ H3824WElmt( 22, 0x02 ) }} ,    // RX SQ1 ACQ Threshold (high)
    {{ H3824WElmt( 23, 0x10 ) }} ,    // RX SQ1 ACQ Threshold (low)
//    {{ H3824WElmt( 24, 0x00 ) }} ,    // RX SQ1 ACQ Read (high)
//    {{ H3824WElmt( 25, 0x00 ) }} ,    // RX SQ1 ACQ Read (low)
    {{ H3824WElmt( 26, 0x00 ) }} ,    // RX SQ1 Data Threshold (high)
    {{ H3824WElmt( 27, 0x00 ) }} ,    // RX SQ1 Data Threshold (low)
//    {{ H3824WElmt( 26, 0x0F ) }} ,    // RX SQ1 Data Threshold (high) (per Harris 10/8/96)
//    {{ H3824WElmt( 27, 0xFF ) }} ,    // RX SQ1 Data Threshold (low) (per Harris 10/8/96)
//    {{ H3824WElmt( 28, 0x00 ) }} ,    // RX SQ1 Data Threshold Read (high)
//    {{ H3824WElmt( 29, 0x00 ) }} ,    // RX SQ1 Data Threshold Read (low)
    {{ H3824WElmt( 30, 0x00 ) }} ,    // RX SQ2 ACQ Threshold (high)
//    {{ H3824WElmt( 31, 0x96 ) }} ,    // Old RX SQ2 ACQ Threshold (low)
    {{ H3824WElmt( 31, 0x90 ) }} ,    // RX SQ2 ACQ Threshold (low)
//    {{ H3824WElmt( 32, 0x00 ) }} ,    // RX SQ2 ACQ Read (high)
//    {{ H3824WElmt( 33, 0x00 ) }} ,    // RX SQ2 ACQ Read (low)
    {{ H3824WElmt( 34, 0xFF ) }} ,    // RX SQ2 Data Threshold (high)
    {{ H3824WElmt( 35, 0xFF ) }} ,    // RX SQ2 Data Threshold (low)
//    {{ H3824WElmt( 34, 0x09 ) }} ,    // RX SQ2 Data Threshold (high) (per Harris 10/8/96)
//    {{ H3824WElmt( 35, 0x80 ) }} ,    // RX SQ2 Data Threshold (low) (per Harris 10/8/96)
//    {{ H3824WElmt( 36, 0x00 ) }} ,    // RX SQ2 Data Read (high)
//    {{ H3824WElmt( 37, 0x00 ) }} ,    // RX SQ2 Data Read (low)
//    {{ H3824WElmt( 38, 0x00 ) }} ,    // RX Signal Quality 8-bit Read

//    {{ H3824WElmt( 39, 0x00 ) }} ,    // Reserved, write 0 (per Harris 10/8/96)

/*
  Per Harris 3/25/98

  Bit 6 - specifies that the length field be interpreted by the 3824 as a
          time measure whose granularity is the period of a BPSK data bit.
          With the DS radios configured for 802.11 this measure is in
          microseconds.
  Bit 3 - changes the integration interval in the receiver such that a higher
          oscillator offset between the transmitter and receiver is
          tolerated.  Practically speaking, this means that cheaper
          oscillators may be used without a performance hit. Still,
          since the integration interval is shorter, the tracking (Data)
          thresholds must be adjusted.  But, in our case, these thresholds
          are railed such that we never lose lock and therefore need not be
          changed.
  Bit 2 - This bit enables a counter used by the CCA circuitry to signal the
          end of a frame based on the received length field.  For a given
          frame, the timer is only activated if the header CRC is good. Once
          activated it will maintain the "busy medium" state of CCA until
          it times out. Furthermore, RX_PE will not reset this timer function
          (but RESET- will).
*/
    {{ H3824WElmt( 39, 0x4c ) }} ,    // see above note
// Per Harris 04/01/98 no register 40 exists
//    {{ H3824WElmt( 40, 0x00 ) }} ,    // Reserved, write 0 (per Harris 10/8/96)

    {{ H3824WElmt( 41, 0x90 ) }} ,    // SFD RX search timer limit (symbols)
    {{ H3824WElmt( 42, 0x0A ) }} ,    // BPSK_SIGNAL
    {{ H3824WElmt( 43, 0x14 ) }} ,    // QPSK_SIGNAL
//    {{ H3824WElmt( 44, 0x00 ) }} ,    // RX'd service field (reserved)
//    {{ H3824WElmt( 45, 0x00 ) }} ,    // RX'd length field (high)
//    {{ H3824WElmt( 46, 0x00 ) }} ,    // RX'd length field (low)
//    {{ H3824WElmt( 47, 0x00 ) }} ,    // RX'd CRC field (high)
//    {{ H3824WElmt( 48, 0x00 ) }} ,    // RX'd CRC field (low)
    {{ H3824WElmt( 49, 0xF3 ) }} ,    // SFD_B1
    {{ H3824WElmt( 50, 0xA0 ) }} ,    // SFD_B0
    {{ H3824WElmt( 51, 0x00 ) }} ,    // TX DS_PLCP_SERVICE
//    {{ H3824WElmt( 52, 0x00 ) }} ,    //
//    {{ H3824WElmt( 53, 0x00 ) }} ,    //
//    {{ H3824WElmt( 54, 0x00 ) }} ,    //
//    {{ H3824WElmt( 55, 0x00 ) }} ,    //
    {{ H3824WElmt( 56, 0x80 ) }}    // TX preamble length (bits)
  }; /* end H3824_Init_Table */

//for (write_reg=0 ; write_reg<WRITEABLE_REGISTERS ; write_reg++) {
//  ptr = (char far*) &astH3824_Init_Table[write_reg];
//  for (attempts_left=ATTEMPTS_PER_REGISTER;attempts_left>0;attempts_left--) {

//    TIR[SERIAL_DEVICE] = SDSEL3;
          // /WRITE and AS are currently asserted, but not /CS

// State machine reset clock (SDCLK pulse with /CS deasserted)
// SDSEL3     asserts the /WR signal to the 3824
// TIR2_SDCP  asserts the Serial Device Clock Auto Pulse
//
// Absence of SDSEL2 asserts the Address signal (phase) to the 3824.
// Absence of SDSEL1 asserts the /CS is DISABLED.
// SDC is a 0 so the value of auto clock pulse will be 1 (high active pulse)
// SDDT is a 0 so the SDDATA pin is driven with the value of the SDD bit
// SDD is a 0 so the value driven onto the SDDATA pin is a 0 (low)
    TIR[SERIAL_DEVICE] = SDSEL3 | TIR2_SDCP;
// negate the Serial Device Clock Auto Pulse (reset complete)
    TIR[SERIAL_DEVICE] = SDSEL3;

  for (attempts_left=kBlockPrgmAttempts ; attempts_left ; attempts_left--) {
    for (write_reg=0 ; write_reg<WRITEABLE_REGISTERS ; write_reg++) {

      ptr = (char far*) &astH3824_Init_Table [write_reg];

// SDSEL3     asserts the /WR signal to the 3824
// SDSEL1     asserts the /CS signal to the 3824
      TIR[SERIAL_DEVICE] = SDSEL3 | SDSEL1;

// Read the address value from the table and write it to the 3824 using the
// fast serial port
      TIR[FST_SER_PRT_CNTL] = ptr[0];   // 3824 address bits
      TIR[FST_SER_PRT_CNTL] = ptr[1];

// SDSEL2     asserts the DATA signal (phase) to the 3824
      TIR[SERIAL_DEVICE] = SDSEL3 + SDSEL2 + SDSEL1;
          // Leave /WRITE and /CS asserted
          // Deassert AS

// Read the data value from the table and write it to the 3824 using the
// fast serial port
      TIR[FST_SER_PRT_CNTL] = ptr[2];   // 3824 data bits
      TIR[FST_SER_PRT_CNTL] = ptr[3];

// The HSP3824 register write is now completed
    }

/////////////////////////////////////////////////////////////////////////////

// Readback to verify that the data was written correctly

// State machine reset clock (SDCLK pulse with /CS deasserted)
// SDSEL3     asserts the /WR signal to the 3824
// TIR2_SDCP  asserts the Serial Device Clock Auto Pulse
//
// Absence of SDSEL2 asserts the Address signal (phase) to the 3824.
// Absence of SDSEL1 asserts the /CS is DISABLED.
// SDC is a 0 so the value of auto clock pulse will be 1 (high active pulse)
// SDDT is a 0 so the SDDATA pin is driven with the value of the SDD bit
// SDD is a 0 so the value driven onto the SDDATA pin is a 0 (low)

    TIR[SERIAL_DEVICE] = SDSEL3 | TIR2_SDCP;
// negate the Serial Device Clock Auto Pulse (reset complete)
   TIR[SERIAL_DEVICE] = SDSEL3;

    for (write_reg=0 ; write_reg<WRITEABLE_REGISTERS ; write_reg++) {

      ptr = (char far*) &astH3824_Init_Table [write_reg];

      TIR[SERIAL_DEVICE] = SDSEL3 | SDSEL1;
          // Assert /WRITE, AS, and /CS

      TIR[FST_SER_PRT_CNTL] = ptr[0];   // 3824 address bits
      TIR[FST_SER_PRT_CNTL] = ptr[1] & 0xDF;  // (7 bits only)

      asm {

// si is used in order to speed up the read process
        push    si
        mov     si,offset TIR+SERIAL_DEVICE

// assert read, address phase, chip select enabled, tristate SDDATA
        mov     byte ptr [si],012h // 0 001 0010b

// assert read, address phase, chip select enabled, SDCLK active, tristate SDDATA
        mov     byte ptr [si],01ah // 0 001 1010b

// bit 7
// assert read, data phase, chip select enabled, SDCLK inactive, tristate SDDATA
        mov     byte ptr [si],032h // 0 011 0010b
        mov     al,byte ptr [si]   // read the SDDATA pin
        shr     al,1               // move SDD into the carry flag
        rcl     ah,1               // move carry flag into ah

// bit 6
// from this point forward the every time the TIR2 register is written it
// will activate the SDCLK pin thus clocking in the data
// assert read, data phase, chip select, SDCLK active, trisate SDDATA
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]   // read the SDDATA pin
        shr     al,1               // move SDD into the carry flag
        rcl     ah,1               // move carry flag into ah

// bit 5
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// bit 4
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// bit 3
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// bit 2
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// bit 1
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// bit 0
        mov     byte ptr [si],03ah // 0 011 1010b
        mov     al,byte ptr [si]
        shr     al,1
        rcl     ah,1

// assert read, data phase, chip select enabled, SDCLK active, tristate SDDATA
        mov     byte ptr [si],03ah // 0 011 1010b

// assert write, address phase, chip select disabled, SDCLK inactive, tristate SDDATA
        mov     byte ptr [si],040h // 0 100 0000b

        mov     return_value, ah
        pop     si

      }

      if (return_value ^= ptr[4]) {
/*
  There are three flavors of the Harris 3824 baseband processor.
  Register 39 does not exist on the HSP3824 but does exist on the HFA3824
  and HFA3824A parts.

  In order to prevent failure on the programming verification of the
  HSP3824 it is assumed that this register is written correctly if it was
  a HFA part and is therefore skipped over.

  NOTE:  This is only necessary if there is a mix of HSP and HFA parts by
         the manufacturer.
*/
		  if ((((Unsigned16*)ptr)[0]) == H3824address(39)) {
          continue ;
        }
        else {
          break ;
        }
      }
    } // End of the read block verification loop
// If the value was zero then the block verified and there is no need to
// ..reprogram the registers.
    if (return_value == 0x00)
      return return_value ;
  }
  return return_value;          // Return error bits if fail


} /* end of initialize_3824() */



//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

// The following function performs the one-time initialization of the
// PLL chip as defined by the constants stSynthesizer_IFR_Counter,
// stSynthesizer_IFN_Counter, and stSynthesizer_RFR_Counter.
//
//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
void initialize_PLL(void) {

// Uses macro PLLRElmt()
//  PLLRElmt (PGM(R16..R20), DivRatio(R15..R1), CTRL(C1.C2))
  const Synth_Data_Type stSynthesizer_IFR_Counter = {
  {PLLRElmt(24,22,0)}
  };
// Uses macro PLLNElmt()
//  PLLNElmt (PGM(N19.N20), DivRatioB(N18..N8), DivRatioA(N7..N1), CTRL(C1.C2))
  const Synth_Data_Type stSynthesizer_IFN_Counter = {
  {PLLNElmt(2,35,0,2)}
  };
// Uses macro PLLRElmt()
//  PLLRElmt (PGM(R16..R20), DivRatio(R15..R1), CTRL(C1.C2))
  const Synth_Data_Type stSynthesizer_RFR_Counter = {
  {PLLRElmt(24,22,1)}
  };


  const far char* ptr;

  // Initialize pointer to IFR counter value
  ptr = (char far*) &stSynthesizer_IFR_Counter;

  TIR[SERIAL_DEVICE] |= TIR2_SDC;    // SDC pin HIGH (Active LOW pulses)

// Assert
  #ifdef IMAX
    // Assert /LE (/HFPE set LOW) to enable clock and data inputs
    TIR[NETWORK_CONTROL] |=HFPE;
  #else
    // Assert /LE (USER4) to enable clock and data inputs
    TIR[USER_PIN_DATA] &= ~TIR29_USERDT4;
  #endif

  TIR[FST_SER_PRT_CNTL] = ptr[0];
  TIR[FST_SER_PRT_CNTL] = ptr[1];
  TIR[FST_SER_PRT_CNTL] = ptr[2];
  TIR[FST_SER_PRT_CNTL] = ptr[3];
  TIR[FST_SER_PRT_CNTL] = ptr[4];

// DeAssert
  #ifdef IMAX
    // Deassert /LE (HFPE set HIGH) to load shift register contents into synthesizer
    TIR[NETWORK_CONTROL] &=~HFPE;      //  Defined in *.hpp as 0x001
  #else
    // Deassert /LE (USER4) to load shift register contents into synthesizer
    TIR[USER_PIN_DATA] |= TIR29_USERDT4;
  #endif

  // Initialize pointer to IFN counter value
  ptr = (char far*) &stSynthesizer_IFN_Counter;
// Assert
  #ifdef IMAX
    // Assert /LE (/HFPE set LOW) to enable clock and data inputs
    TIR[NETWORK_CONTROL] |=HFPE;
  #else
    // Assert /LE (USER4) to enable clock and data inputs
    TIR[USER_PIN_DATA] &= ~TIR29_USERDT4;
  #endif

  TIR[FST_SER_PRT_CNTL] = ptr[0];
  TIR[FST_SER_PRT_CNTL] = ptr[1];
  TIR[FST_SER_PRT_CNTL] = ptr[2];
  TIR[FST_SER_PRT_CNTL] = ptr[3];
  TIR[FST_SER_PRT_CNTL] = ptr[4];

// DeAssert
  #ifdef IMAX
    // Deassert /LE (HFPE set HIGH) to load shift register contents into synthesizer
    TIR[NETWORK_CONTROL] &=~HFPE;      //  Defined in *.hpp as 0x001
  #else
    // Deassert /LE (USER4) to load shift register contents into synthesizer
    TIR[USER_PIN_DATA] |= TIR29_USERDT4;
  #endif

  // Initialize pointer to RFR counter value
  ptr = (char far*) &stSynthesizer_RFR_Counter;

// Assert
  #ifdef IMAX
    // Assert /LE (/HFPE set LOW) to enable clock and data inputs
    TIR[NETWORK_CONTROL] |=HFPE;
  #else
    // Assert /LE (USER4) to enable clock and data inputs
    TIR[USER_PIN_DATA] &= ~TIR29_USERDT4;
  #endif

  TIR[FST_SER_PRT_CNTL] = ptr[0];
  TIR[FST_SER_PRT_CNTL] = ptr[1];
  TIR[FST_SER_PRT_CNTL] = ptr[2];
  TIR[FST_SER_PRT_CNTL] = ptr[3];
  TIR[FST_SER_PRT_CNTL] = ptr[4];

// DeAssert
  #ifdef IMAX
    // Deassert /LE (HFPE set HIGH) to load shift register contents into synthesizer
    TIR[NETWORK_CONTROL] &=~HFPE;      //  Defined in *.hpp as 0x001
  #else
    // Deassert /LE (USER4) to load shift register contents into synthesizer
    TIR[USER_PIN_DATA] |= TIR29_USERDT4;
  #endif

  TIR[SERIAL_DEVICE] &= ~TIR2_SDC;    // SDC pin LOW

}; /* end of initialize_PLL() */

//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#endif /* DS_RADIO_CPP */
