#ifndef TYPES_HPP
#define TYPES_HPP

/*
/*
 *
 * Radio API source module for PCnet-MOBILE 802.11 protocol firmware.
 *
 * Copyright (c) 1996, 1997, 1998 Advanced Micro Devices, Inc.
 * All rights reserved.
 *
 * Version 2.0
 *
 * Controller & Switch Development (CSD)
 *
 * Module Name: types.hpp
 *
 * Module Names:
 *    Unsigned8
 *    Unsigned16
 *    Unsigned24
 *    Unsigned24 sub_24_bits(Unsigned24, Unsigned24)
 *    Unsigned24 add_24_bits(Unsigned24, Unsigned24)
 *    int gt_24_bits(Unsigned24, Unsigned24)
 *    int lt_24_bits(Unsigned24, Unsigned24)
 *    Unsigned32
 *    Boolean
 *    ProcStatusType
 *    Offset_Type
 *    HardwareTimeUnits
 *    BackoffTimeUnits
 *    Unsigned48
 *    Unsigned64
 *    Unsigned64_div_t
 *
 * Globals Accessed:
 *
 * Callers:
 *    Anything that uses one of the above types.
 *
 * Called routines:
 *    memcpy
 *    memset
 *    memcmp
 *
 * Functional Description:
 *    This file contains the definitions of all the global types used in the
 *  firmware. Unsigned8, Unsigned16, Unsigned32 are integral types provided by
 *  most compilers but are different for different compilers and architectures.
 *  Centralizing these type declarations and using them in the code clarifies
 *  the size and type of most variables and provides a single point of change
 *  for porting to other memory models, architectures or compilers.
 *
 *  Unsigned8
 *     An unsigned 8-bit quantity. Used wherever an octet is needed.
 *
 *  Unsigned16
 *     An unsigned 16-bit quantity.
 *
 *  Unsigned24
 *     An unsigned 24-bit quantity. The free running counter and the power
 *     down counter in the SUTRO are 24-bit values. This type actually uses a
 *     dword, but the manipulation routines defined below use the lower 24
 *     bits as an unsigned number.
 *
 *    Unsigned24 sub_24_bits(Unsigned24, Unsigned24)
 *      Subtracts the second parameter from the first and returns the lower
 *      24-bits. The upper byte of the dword result is always 0.
 *
 *    Unsigned24 add_24_bits(Unsigned24, Unsigned24)
 *      Adds the second parameter to the first and returns the lower
 *      24-bits. The upper byte of the dword result is always 0.
 *
 *    int gt_24_bits(Unsigned24, Unsigned24)
 *      Returns 0 if second parameter is less than or equal to the first
 *      parameter.
 *
 *    int lt_24_bits(Unsigned24, Unsigned24)
 *      Returns 0 if first parameter is less than or equal to the second
 *      parameter.
 *
 *  Unsigned32
 *     An unsigned 32-bit quantity.
 *
 *  Boolean
 *    An enumerated type that can take on the values TRUE or FALSE. Used for
 *    logical true or false information.
 *
 *  ProcStatusType
 *    Type of the variable used to store the processor status word. Is a
 *    16-bit quantity for the SUTRO.
 *
 *  Offset_Type
 *    Type of the pointer offsets in the Rx and Tx descriptors and size of
 *    the receive and transmit buffers. Is a 16-bit quantity in the current
 *    version of the firmware.
 *
 *  HardwareTimeUnits
 *    Type used for hardware timers in the SUTRO. Is a 16-bit quantity in the
 *    SUTRO.
 *
 *  Unsigned48
 *    Is not an integral type. This class provides 48-bit entities that can
 *    be used and manipulated as required for MAC addresses. An Unsigned48
 *    quantity is a ordered string of 6 bytes, not numeric quantity with
 *    significance.
 *
 *  Unsigned64
 *    Is not an integral class. This class provides 64-bit entities that can
 *    be used and manipulated as required for TSF values by the MAC.
 *
 *  Unsigned64_div_t
 *    Is not an integral class. This class provides divide functions (similar
 *    to the div_t and ldiv_t sturctures in Borland C++ for int and long) for
 *    the Unsigned64 class.
 *
 *    div_by_k
 *      A special divide function provided to optimize the division of TSF
 *      values by Unsigned16 variables containing kusec values.
 *
 * Comments:
 *
 * Diagnostics:
 *
 */

#include <stdlib.h>
#include <mem.h>

/* The following macros allow Borland C++ to inline the associated routines */
#define memcpy __memcpy__
#define memset __memset__
#define memcmp __memcmp__

typedef unsigned char  Unsigned8;
typedef unsigned short Unsigned16;
typedef unsigned long  Unsigned24;
typedef unsigned long  Unsigned32;

inline Unsigned24 sub_24_bits(const Unsigned24& dword1, const Unsigned24& dword2)
/*
 *  Subtracts the second parameter from the first and returns the lower 24-bits.
 *  The upper byte of the dword result is always 0.
 */
{
  return Unsigned24((dword1 - dword2) & 0xffffffL);
} /* end sub_24_bits */

inline Unsigned24 add_24_bits(const Unsigned24& dword1, const Unsigned24& dword2)
/*
 *  Adds the second parameter to the first and returns the lower 24-bits. The
 *  upper byte of the dword result is always 0.
 */
{
  return Unsigned24((dword1 + dword2) & 0xffffffL);
} /* end add_24_bits */

inline int gt_24_bits(const Unsigned24& dword1, const Unsigned24& dword2)
/*
 *  Returns 0 if second parameter is less than or equal to the first parameter.
 *  Subtracts the first parameter from the second and returns non-zero if the
 *  result is negative (24-bits only).
 */
{
  return (int((dword2 - dword1) >> 16) & 0x80);
} /* end gt_24_bits */

inline int lt_24_bits(const Unsigned24& dword1, const Unsigned24& dword2)
/*
 *  Returns 0 if first parameter is greater than or equal to the second
 *  parameter. Subtracts the second parameter from the first and returns
 *  non-zero if the result is negative (24-bits only).
 */
{
  return (int((dword1 - dword2) >> 16) & 0x80);
} /* end lt_24_bits */

#ifndef FALSE
  typedef enum {FALSE = 0, TRUE = 1} Boolean;
#else
  typedef Unsigned8 Boolean;
#endif /* FALSE */

/* Processor status word type, used for disable and restore interrupts */
typedef Unsigned16 ProcStatusType;

/* Offset Type is used in rx and tx descriptors */
typedef Unsigned16 Offset_Type;

/*  each is a unit of Hardware(hdw) Timer value */
typedef Unsigned16 HardwareTimeUnits;

/*  each unit is 2 units of Hardware(hdw) Timer value */
typedef Unsigned16 DwellTimeUnits;

/*  each unit is 4 units of Hardware(hdw) Timer value */
typedef Unsigned16 BackoffTimeUnits;

class Unsigned64;

static const unsigned char MULTICAST_BIT = 0x01;
static const unsigned char LOCAL_ADMIN_BIT = 0x02;

class Unsigned48
/*
 *  Is not an integral type. This class provides 48-bit entities that can
 *  be used and manipulated as required for MAC addresses. An Unsigned48
 *  quantity is a ordered string of 6 bytes, not numeric quantity with
 *  significance.
 */
{
  private:
    /*
     *  allows Unsigned64 methods to operate on an Unsigned48 object as if
     *  they were Unsigned48 class methods. Used to generate pseudo-random
     *  BSSIDs.
     */
    friend Unsigned64;

  protected:
    /*
     *  Union declaration allows bytes to be manipulated as words or a
     *  dword+.
     */
    union {
      unsigned char bytes[6];        // big endian
      unsigned short words[3];      // big endian
      unsigned long first_dword;      // big endian
    } value;

  public:
    Unsigned48()
    /* default constructor */
    {
    } /* end Unsigned48 */

    Unsigned48(const unsigned long& first_24,
      const unsigned long& second_24)
    /*
     *  Create an Unsigned48 quantity from 2 Unsigned24 quantities. Treat
     *  the Unsigned24 quantities as streams of bytes with the most
     *  significant byte first.
     */
    {
      value.bytes[0] = ((unsigned char*) &first_24)[2];
      value.bytes[1] = ((unsigned char*) &first_24)[1];
      value.bytes[2] = ((unsigned char*) &first_24)[0];
      value.bytes[3] = ((unsigned char*) &second_24)[2];
      value.bytes[4] = ((unsigned char*) &second_24)[1];
      value.bytes[5] = ((unsigned char*) &second_24)[0];
    } /* end Unsigned48 */

    operator char*() const {
      return((char*) value.bytes);
    } /* end operator char*() */

    const Unsigned48& operator =(const Unsigned48& newItem) {
      (void) memcpy((char *) value.bytes, (char *) newItem.value.bytes, 6);

      return (*this);
    } /* assign newItem */

    const Unsigned48& operator =(const char* bytes) {
      (void) memcpy((char *) value.bytes, bytes, 6);

      return (*this);
    } /* end Unsigned48 */

    const int operator !=(const Unsigned48& newItem) const {
//      return ((value.words[0] != newItem.value.words[0]) ||
//        (value.words[1] != newItem.value.words[1]) ||
//        (value.words[2] != newItem.value.words[2]));
      return ((value.words[0] ^ newItem.value.words[0]) |
          (value.words[1] ^ newItem.value.words[1]) |
          ~(~value.words[2] ^ newItem.value.words[2]));
    } /* compare for not-equal newItem */

    const int operator ==(const Unsigned48& newItem) const {
//      return ((value.words[0] == newItem.value.words[0]) &&
//        (value.words[1] == newItem.value.words[1]) &&
//        (value.words[2] == newItem.value.words[2]));
      return (((value.words[0] ^ newItem.value.words[0]) |
          (value.words[1] ^ newItem.value.words[1]) |
          ~(~value.words[2] ^ newItem.value.words[2])) == 0);
    } /* compare for equal newItem */

    const Unsigned8 first_byte() const {
      return (value.bytes[0]);
    } /* end first_byte */

    const Boolean is_multicast() const {
      return Boolean(value.bytes[0] & MULTICAST_BIT);
    } /* end is_multicast */

    const Unsigned8 hash() const
    /* hash function used by Host_Rx to search the src_entry table. */
     {
      unsigned char result = value.bytes[0];
      result = (result << 1) + value.bytes[1];
      result = (result << 1) + value.bytes[2];
      result = (result << 1) + value.bytes[3];
      result = (result << 1) + value.bytes[4];
      result = (result << 1) + value.bytes[5];

      return Unsigned8(result);
    } /* end hash */
}; /* end class Unsigned48 */

class Unsigned64_div_t;

class Unsigned64 {
  private:
    friend Unsigned64_div_t;

    friend const Unsigned64_div_t div(
      const Unsigned64& numerator, Unsigned16 divisor);

    friend const Unsigned64_div_t div_by_k(
      const Unsigned64& numerator, Unsigned16 divisor);

  protected:
    union {
      unsigned char bytes[8];
      unsigned short words[4];
      unsigned long dwords[2];
    } value;

  public:
    Unsigned64() {
    } /* end Unsigned64 */

    Unsigned64(const unsigned long& least_sig_32)
    {
      value.dwords[0] = least_sig_32;
      value.dwords[1] = 0L;
    } /* end Unsigned64 */

    Unsigned64(const unsigned long& first_32,
      const unsigned long& second_32)
    /*
     *  Create an Unsigned64 quantity from 2 Unsigned32 quantities. Treat
     *  the first dword as the most significiate dword and the second dword
     *  as the least significant dword of the 64-bit result.
     */
    {
      value.dwords[1] = first_32;
      value.dwords[0] = second_32;
    } /* end Unsigned64 */

    Unsigned64(const Unsigned64& newItem) {
      (void) memcpy(value.bytes, newItem.value.bytes, 8);
    } /* assign newItem */

    operator Unsigned32() const {
      return Unsigned32(value.dwords[0]);
    } /* explicit type conversion to Unsigned32 */

    operator Unsigned16() const {
      return Unsigned16(value.words[0]);
    } /* explicit type conversion to Unsigned16 */

    const Unsigned64& operator =(const Unsigned64& newItem) {
      (void) memcpy(value.bytes, newItem.value.bytes, 8);

      return *this;
    } /* assign newItem */

    const Unsigned64& operator +=(const Unsigned64& newItem)
    /*
     *  Add a 64-bit value to the given 64-bit value. The function
     *  does the addition as the sum of two sets of dwords. Carry
     *  propogation is done by checking the most significant bit
     *  in the lower dwords of the two values being added.
     */
    {
      if (char(value.bytes[3] ^ newItem.value.bytes[3]) < char(0)) {
        /* sometimes carry to be generated */

        value.dwords[0] += newItem.value.dwords[0];
        if (char(value.bytes[3]) >= char(0)) {
          /* carry generated */

          value.dwords[1] += 1L + newItem.value.dwords[1];
        } else {
          /* no carry generated */

          value.dwords[1] += newItem.value.dwords[1];
        } /* end if */
      } else if (char(value.bytes[3] & newItem.value.bytes[3]) < char(0)) {
        /* always carry to be generated */

        value.dwords[0] += newItem.value.dwords[0];
        value.dwords[1] += 1L + newItem.value.dwords[1];
      } else {
        /* never carry to be generated */

        value.dwords[0] += newItem.value.dwords[0];
        value.dwords[1] += newItem.value.dwords[1];
      } /* end if */

      return *this;
    } /* add and assign Unsigned64 */

    const Unsigned64& operator +=(const Unsigned32& newItem)
    /*
     *  Add a 32-bit value to the given 64-bit value. The function
     *  does the addition as the sum of a dword to the lower dword of the
     *  64-bit value. Carry propogation is done by checking the most
     *  significant bit in newItem and the lower dword of the 64-bit value.
     */
    {
      if (char(value.bytes[3] ^ ((char*) &newItem)[3]) < char(0)) {
        /* sometimes carry to be generated */

        value.dwords[0] += (unsigned long) newItem;
        if (char(value.bytes[3]) >= char(0)) {
          /* carry generated */

          value.dwords[1] += 1L;
        } /* end if */
      } else if (char(value.bytes[3] & ((char*) &newItem)[3]) < char(0)) {
        /* always carry to be generated */

        value.dwords[0] += (unsigned long) newItem;
        value.dwords[1] += 1L;
      } else {
        /* never carry to be generated */

        value.dwords[0] += (unsigned long) newItem;
      } /* end if */

      return *this;
    } /* add and assign Unsigned32 */

    const Unsigned64& operator +=(const Unsigned16& newItem)
    /*
     *  Add a 16-bit value to the given 64-bit value. The function
     *  does the addition as the sum of a dword to the lower dword of the
     *  64-bit value. Carry propogation is done by checking the most
     *  significant bit of the lower dword of the 64-bit value and result.
     */
    {
      if (char(value.bytes[3]) < char(0)) {
        /* sometimes carry */

        value.dwords[0] += (unsigned long) newItem;
        if (char(value.bytes[3]) >= char(0)) {
          /* carry generated */

          value.dwords[1] += 1L;
        } /* end if */
      } else {
        /* never carry to be generated */

        value.dwords[0] += (unsigned long) newItem;
      } /* end if */

      return *this;
    } /* add and assign Unsigned16 */

    const Unsigned64& operator -=(const Unsigned64& newItem)
    /*
     *  Subtract a 64-bit value from the given 64-bit value. The function
     *  does the subtraction as the difference of two sets of dwords. Carry
     *  propogation is done by comparing the lower dwords first.
     */
    {
      if (value.dwords[0] >= newItem.value.dwords[0]) {
        /* no borrow to be generated */

        value.dwords[0] -= newItem.value.dwords[0];
        value.dwords[1] -= newItem.value.dwords[1];
      } else {
        /* always borrow to be generated */

        value.dwords[0] -= newItem.value.dwords[0];
        value.dwords[1] =
          value.dwords[1] - newItem.value.dwords[1] - 1L;
      } /* end if */

      return *this;
    } /* subtract and assign Unsigned64 */

    const Unsigned64& operator -=(const Unsigned32& newItem)
    /*
     *  Subtract a 32-bit value from the given 64-bit value. The function
     *  does the subtraction as the difference of a dword from the lower
     *  dword of the 64-bit value. Carry propogation is done by comparing
     *  the lower dword with the passed dword first.
     */
    {
      if (value.dwords[0] >= newItem) {
        /* no borrow to be generated */

        value.dwords[0] -= newItem;
      } else {
        /* always borrow to be generated */

        value.dwords[0] -= newItem;
        value.dwords[1] -= 1L;
      } /* end if */

      return *this;
    } /* subtract and assign Unsigned32 */

    const Unsigned64& operator -=(const Unsigned16& newItem)
    /*
     *  Subtract a 16-bit value from the given 64-bit value. The function
     *  does the subtraction as the difference of a dword from the lower
     *  dword of the 64-bit value. Borrow into the upper dword is only
     *  generated if the least significant word is < newItem and word[1]
     *  is zero.
     */
    {
      if (value.words[0] >= newItem) {
        /* never generate a borrow */

        value.words[0] -= newItem;
      } else if (value.words[1] > 0) {
        /* never generate a borrow from second word */

        value.words[0] -= newItem;
        value.words[1] -= 1;
      } else {
        /* always borrow from upper dword */

        value.dwords[0] -= Unsigned32(newItem);
        value.dwords[1] -= 1L;
      } /* end if */

      return *this;
    } /* subtract and assign Unsigned16 */

    const Unsigned64 operator +(const Unsigned64& newItem) const {
      Unsigned64 result(*this);

      return (result += newItem);
    } /* add Unsigned64 */

    const Unsigned64 operator +(const Unsigned32& newItem) const {
      Unsigned64 result(*this);

      return (result += newItem);
    } /* add Unsigned32 */

    const Unsigned64 operator +(const Unsigned16& newItem) const {
      Unsigned64 result(*this);

      return (result += newItem);
    } /* add Unsigned16 */

    const Unsigned64 operator -(const Unsigned64& newItem) const {
      Unsigned64 result(*this);

      return (result -= newItem);
    } /* subtract Unsigned64 */

    const Unsigned64 operator -(const Unsigned32& newItem) const {
      Unsigned64 result(*this);

      return (result -= newItem);
    } /* subtract Unsigned32 */

    const Unsigned64 operator -(const Unsigned16& newItem) const {
      Unsigned64 result(*this);

      return (result -= newItem);
    } /* subtract Unsigned16 */

  protected:
    union rem_union {
      unsigned short words[2];
      unsigned long dword;
    };

  public:

    const Unsigned16 operator %(const Unsigned16& divisor) const
    /*
     *  Use long division method with each "digit" being 16-bits to
     *  calculate remainder.
     */
    {
      union rem_union rem;

      rem.words[1] = value.words[3] % divisor;

      rem.words[0] = value.words[2];
      rem.words[1] = (unsigned short) (rem.dword % divisor);

      rem.words[0] = value.words[1];
      rem.words[1] = (unsigned short) (rem.dword % divisor);

      rem.words[0] = value.words[0];
      return Unsigned16((unsigned short) (rem.dword % divisor));
    } /* modulus by Unsigned16 */

    const int operator !=(const Unsigned64& newItem) const {
      return ((value.dwords[0] != newItem.value.dwords[0]) ||
        (value.dwords[1] != newItem.value.dwords[1]));
    } /* compare for not-equal newItem */

    const int operator ==(const Unsigned64& newItem) const {
      return ((value.dwords[0] == newItem.value.dwords[0]) &&
        (value.dwords[1] == newItem.value.dwords[1]));
    } /* compare for equal newItem */

    const int operator <(const Unsigned32& newItem) const {
      return ((value.dwords[1] != 0L) ?
                      0 : (value.dwords[0] < newItem));
    } /* compare for < newItem */

    const int operator <(const Unsigned64& newItem) const {
      return ((value.dwords[1] == newItem.value.dwords[1]) ?
        (value.dwords[0] < newItem.value.dwords[0]) :
        (value.dwords[1] < newItem.value.dwords[1]));
    } /* compare for < newItem */

    const int operator <=(const Unsigned64& newItem) const {
      return ((value.dwords[1] == newItem.value.dwords[1]) ?
        (value.dwords[0] <= newItem.value.dwords[0]) :
        (value.dwords[1] <= newItem.value.dwords[1]));
    } /* compare for <= newItem */

    const int operator >(const Unsigned32& newItem) const {
      return ((value.dwords[1] != 0L) ?
                      1 : (value.dwords[0] > newItem));
    } /* compare for > newItem */

    const int operator >(const Unsigned64& newItem) const {
      return ((value.dwords[1] == newItem.value.dwords[1]) ?
        (value.dwords[0] > newItem.value.dwords[0]) :
        (value.dwords[1] > newItem.value.dwords[1]));
    } /* compare for > newItem */

    const int operator >=(const Unsigned64& newItem) const {
      return ((value.dwords[1] == newItem.value.dwords[1]) ?
        (value.dwords[0] >= newItem.value.dwords[0]) :
        (value.dwords[1] >= newItem.value.dwords[1]));
    } /* compare for >= newItem */

    const Unsigned48 generate_BSSID() const
    /*
     *  Return the least significant 48-bits of the 64-bit value to
     *  generate a non-multicast locally administered MAC address.
     */
    {
      Unsigned48 result;

      result.value.first_dword = value.dwords[0];
      result.value.words[2] = value.words[2];

      result.value.bytes[0] &= ~MULTICAST_BIT;
      result.value.bytes[0] |= LOCAL_ADMIN_BIT;

      return (result);
    } /* end generate_BSSID */

}; /* end class Unsigned64 */


class Unsigned64_div_t {
  public:
    Unsigned64  quot;
    Unsigned32  rem;

    friend const Unsigned64_div_t div(
      const Unsigned64& numerator, Unsigned16 divisor);

    friend inline const Unsigned64_div_t div_by_k(
      const Unsigned64& numerator, Unsigned16 divisor);
}; /* end class Unsigned64_div_t */

#endif /* TYPES_HPP */



